@@ -161,6 +161,14 @@ brasero_burn_powermanagement (BraseroBurn *self,
 		brasero_uninhibit_suspend (priv->appcookie); 
 }
 
+/**
+ * brasero_burn_new:
+ *
+ *  Creates a new #BraseroBurn object.
+ *
+ * Return value: a #BraseroBurn object.
+ **/
+
 BraseroBurn *
 brasero_burn_new ()
 {
@@ -272,7 +280,7 @@ brasero_burn_eject (BraseroBurn *self,
 {
 	guint counter = 0;
 
-	brasero_drive_eject (drive, TRUE, error);
+	brasero_drive_eject (drive, TRUE, NULL);
 
 	/* sleep some time and see what happened */
 	brasero_burn_sleep (self, 500);
@@ -299,7 +307,7 @@ brasero_burn_eject (BraseroBurn *self,
 		}
 
 		BRASERO_BURN_LOG ("Retrying ejection");
-		brasero_drive_eject (drive, TRUE, error);
+		brasero_drive_eject (drive, TRUE, NULL);
 		brasero_burn_sleep (self, 500);
 	}
 
@@ -536,7 +544,7 @@ static BraseroBurnResult
 brasero_burn_lock_src_media (BraseroBurn *burn,
 			     GError **error)
 {
-	gchar *failure;
+	gchar *failure = NULL;
 	BraseroMedia media;
 	BraseroMedium *medium;
 	BraseroBurnResult result;
@@ -1113,17 +1121,8 @@ brasero_burn_unlock_dest_media (BraseroBurn *burn,
 	brasero_drive_unlock (priv->dest);
 
 	if (!BRASERO_BURN_SESSION_EJECT (priv->session)) {
-		if (priv->dest) {
-			GDrive *gdrive;
-
-			gdrive = brasero_drive_get_gdrive (priv->dest);
-
-			/* reprobe the contents of the drive system wide */
-			g_drive_poll_for_media (gdrive, NULL, NULL, NULL);
-			g_object_unref (gdrive);
-
+		if (priv->dest)
 			brasero_drive_reprobe (priv->dest);
-		}
 	}
 	else
 		brasero_drive_eject (priv->dest, FALSE, error);
@@ -1190,6 +1189,20 @@ brasero_burn_action_changed (BraseroTask *task,
 	brasero_burn_action_changed_real (burn, action);
 }
 
+/**
+ * brasero_burn_get_action_string:
+ * @burn: a #BraseroBurn
+ * @action: a #BraseroBurnAction
+ * @string: a #gchar **
+ *
+ * This function returns the current action (in @string)  of
+ * an ongoing operation performed by @burn.
+ * @action is used to set a default string in case there was
+ * no string set by the backend to describe the current
+ * operation.
+ *
+ **/
+
 void
 brasero_burn_get_action_string (BraseroBurn *burn,
 				BraseroBurnAction action,
@@ -1209,6 +1222,24 @@ brasero_burn_get_action_string (BraseroBurn *burn,
 							    string);
 }
 
+/**
+ * brasero_burn_status:
+ * @burn: a #BraseroBurn
+ * @media: a #BraseroMedia or NULL
+ * @isosize: a #goffset or NULL
+ * @written: a #goffset or NULL
+ * @rate: a #guint64 or NULL
+ *
+ * Returns various information about the current operation 
+ * in @media (the current media type being burnt),
+ * @isosize (the size of the data being burnt), @written (the
+ * number of bytes having been written so far) and @rate
+ * (the speed at which data are written).
+ *
+ * Return value: a #BraseroBurnResult. BRASERO_BURN_OK if there is
+ * an ongoing operation; BRASERO_BURN_NOT_READY otherwise.
+ **/
+
 BraseroBurnResult
 brasero_burn_status (BraseroBurn *burn,
 		     BraseroMedia *media,
@@ -1701,7 +1732,7 @@ start:
 	return BRASERO_BURN_ERR;
 }
 
-/* FIXME: for the moment we don't allow for mixed CD type */
+/* FIXME: at the moment we don't allow for mixed CD type */
 static BraseroBurnResult
 brasero_burn_run_tasks (BraseroBurn *burn,
 			gboolean erase_allowed,
@@ -1968,11 +1999,12 @@ brasero_burn_check_session_consistency (BraseroBurn *burn,
 
 	/* save then wipe out flags from session to check them one by one */
 	flags = brasero_burn_session_get_flags (priv->session);
-	brasero_burn_session_remove_flag (priv->session, flags);
+	brasero_burn_session_set_flags (BRASERO_BURN_SESSION (priv->session), BRASERO_BURN_FLAG_NONE);
 
 	result = brasero_burn_session_get_burn_flags (priv->session,
 						      &supported,
 						      &compulsory);
+
 	if (result != BRASERO_BURN_OK)
 		return result;
 
@@ -2264,6 +2296,20 @@ brasero_burn_record_session (BraseroBurn *burn,
 	return result;
 }
 
+/**
+ * brasero_burn_check:
+ * @burn: a #BraseroBurn
+ * @session: a #BraseroBurnSession
+ * @error: a #GError
+ *
+ * Checks the integrity of a medium according to the parameters
+ * set in @session. The medium must be inserted in the #BraseroDrive
+ * set as the source of a #BraseroTrackDisc track inserted in @session.
+ *
+ * Return value: a #BraseroBurnResult. The result of the operation. 
+ * BRASERO_BURN_OK if it was successful.
+ **/
+
 BraseroBurnResult
 brasero_burn_check (BraseroBurn *self,
 		    BraseroBurnSession *session,
@@ -2329,11 +2375,9 @@ brasero_burn_same_src_dest_image (BraseroBurn *self,
 {
 	gchar *toc = NULL;
 	gchar *image = NULL;
-	BraseroTrackImage *track;
 	GError *ret_error = NULL;
 	BraseroBurnResult result;
 	BraseroBurnPrivate *priv;
-	BraseroImageFormat format;
 	BraseroTrackType *output = NULL;
 
 	/* we can't create a proper list of tasks here since we don't know the
@@ -2343,33 +2387,27 @@ brasero_burn_same_src_dest_image (BraseroBurn *self,
 
 	/* get the first possible format */
 	output = brasero_track_type_new ();
-	brasero_track_type_set_has_image (output);
+	result = brasero_burn_session_get_tmp_image_type_same_src_dest (priv->session, output);
 
-	format = BRASERO_IMAGE_FORMAT_CDRDAO;
-	for (; format != BRASERO_IMAGE_FORMAT_NONE; format >>= 1) {
-		brasero_track_type_set_image_format (output, format);
-		result = brasero_burn_session_output_supported (priv->session,
-								output);
-		if (result == BRASERO_BURN_OK)
-			break;
-	}
-	brasero_track_type_free (output);
-
-	if (format == BRASERO_IMAGE_FORMAT_NONE) {
+	if (result != BRASERO_BURN_OK) {
+		brasero_track_type_free (output);
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
 			     _("No format for the temporary image could be found"));
-		return BRASERO_BURN_ERR;
+		return result;
 	}
 
-	/* get a new output. Also ask for both */
+	/* Save the flags for later */
 	brasero_burn_session_push_settings (priv->session);
+
+	/* get a new output. Also ask for both */
 	result = brasero_burn_session_get_tmp_image (priv->session,
-						     format,
+						     brasero_track_type_get_image_format (output),
 						     &image,
 						     &toc,
 						     &ret_error);
+
 	while (result != BRASERO_BURN_OK) {
 		gboolean is_temp;
 
@@ -2377,7 +2415,7 @@ brasero_burn_same_src_dest_image (BraseroBurn *self,
 		||  (ret_error->code != BRASERO_BURN_ERROR_DISK_SPACE
 		&&   ret_error->code != BRASERO_BURN_ERROR_PERMISSION)) {
 			g_propagate_error (error, ret_error);
-			return result;
+			goto end;
 		}
 
 		/* That's an imager (outputs an image to the disc) so that means
@@ -2401,27 +2439,30 @@ brasero_burn_same_src_dest_image (BraseroBurn *self,
 		g_error_free (ret_error);
 		ret_error = NULL;
 
-		if (result != BRASERO_BURN_OK)
-			return result;
+		if (result != BRASERO_BURN_OK) 
+			goto end;
 
 		/* retry */
 		result = brasero_burn_session_get_tmp_image (priv->session,
-							     format,
+							     brasero_track_type_get_image_format (output),
 							     &image,
 							     &toc,
 							     &ret_error);
 	}
 
 	/* some, like cdrdao, can't overwrite the files */
-	g_remove (image);
-	g_remove (toc);
+	if (image)
+		g_remove (image);
+
+	if (toc)
+		g_remove (toc);
 
 	result = brasero_burn_session_set_image_output_full (priv->session,
-							     format,
+							     brasero_track_type_get_image_format (output),
 							     image,
 							     toc);
 	if (result != BRASERO_BURN_OK)
-		return result;
+		goto end;
 
 	/* lock drive */
 	result = brasero_burn_lock_src_media (self, error);
@@ -2440,20 +2481,17 @@ brasero_burn_same_src_dest_image (BraseroBurn *self,
 	if (result != BRASERO_BURN_OK)
 		goto end;
 
-	track = brasero_track_image_new ();
-	brasero_track_image_set_source (track, image, toc, format);
-	brasero_burn_session_add_track (priv->session, BRASERO_TRACK (track), NULL);
-
-	/* It's good practice to unref the track afterwards as we don't need it
-	 * anymore. BraseroBurnSession refs it. */
-	g_object_unref (track);
+	/* There should be a track at the top of the session stack
+	 * so no need to create a new one */
 
 end:
 	g_free (image);
 	g_free (toc);
 
-	brasero_burn_session_pop_settings (priv->session);
+	if (output)
+		brasero_track_type_free (output);
 
+	brasero_burn_session_pop_settings (priv->session);
 	return result;
 }
 
@@ -2531,6 +2569,19 @@ again:
 	return BRASERO_BURN_OK;
 }
 
+/**
+ * brasero_burn_record:
+ * @burn: a #BraseroBurn
+ * @session: a #BraseroBurnSession
+ * @error: a #GError
+ *
+ * Burns or creates a disc image according to the parameters
+ * set in @session.
+ *
+ * Return value: a #BraseroBurnResult. The result of the operation. 
+ * BRASERO_BURN_OK if it was successful.
+ **/
+
 BraseroBurnResult 
 brasero_burn_record (BraseroBurn *burn,
 		     BraseroBurnSession *session,
@@ -2684,6 +2735,20 @@ brasero_burn_blank_real (BraseroBurn *burn, GError **error)
 	return result;
 }
 
+/**
+ * brasero_burn_blank:
+ * @burn: a #BraseroBurn
+ * @session: a #BraseroBurnSession
+ * @error: a #GError
+ *
+ * Blanks a medium according to the parameters
+ * set in @session. The medium must be inserted in the #BraseroDrive
+ * set with brasero_burn_session_set_burner ().
+ *
+ * Return value: a #BraseroBurnResult. The result of the operation. 
+ * BRASERO_BURN_OK if it was successful.
+ **/
+
 BraseroBurnResult
 brasero_burn_blank (BraseroBurn *burn,
 		    BraseroBurnSession *session,
@@ -2751,6 +2816,19 @@ end:
 	return result;
 }
 
+/**
+ * brasero_burn_cancel:
+ * @burn: a #BraseroBurn
+ * @protect: a #gboolean
+ *
+ * Cancels any ongoing operation. If @protect is TRUE then
+ * cancellation will not take place for a "critical" task, a task whose interruption
+ * could damage the medium or the drive.
+ *
+ * Return value: a #BraseroBurnResult. The result of the operation. 
+ * BRASERO_BURN_OK if it was successful.
+ **/
+
 BraseroBurnResult
 brasero_burn_cancel (BraseroBurn *burn, gboolean protect)
 {
