@@ -137,6 +137,23 @@ brasero_burn_caps_get_blanking_flags_real (BraseroBurnCaps *caps,
 	return BRASERO_BURN_OK;
 }
 
+/**
+ * brasero_burn_session_get_blank_flags:
+ * @session: a #BraseroBurnSession
+ * @supported: a #BraseroBurnFlag
+ * @compulsory: a #BraseroBurnFlag
+ *
+ * Given the various parameters stored in @session,
+ * stored in @supported and @compulsory, the flags
+ * that can be used (@supported) and must be used
+ * (@compulsory) when blanking the medium in the
+ * #BraseroDrive (set with brasero_burn_session_set_burner ()).
+ *
+ * Return value: a #BraseroBurnResult.
+ * BRASERO_BURN_OK if the retrieval was successful.
+ * BRASERO_BURN_ERR otherwise.
+ **/
+
 BraseroBurnResult
 brasero_burn_session_get_blank_flags (BraseroBurnSession *session,
 				      BraseroBurnFlag *supported,
@@ -232,6 +249,20 @@ brasero_burn_caps_can_blank_real (BraseroBurnCaps *self,
 	return BRASERO_BURN_NOT_SUPPORTED;
 }
 
+/**
+ * brasero_burn_session_can_blank:
+ * @session: a #BraseroBurnSession
+ *
+ * Given the various parameters stored in @session, this
+ * function checks whether the medium in the
+ * #BraseroDrive (set with brasero_burn_session_set_burner ())
+ * can be blanked.
+ *
+ * Return value: a #BraseroBurnResult.
+ * BRASERO_BURN_OK if it is possible.
+ * BRASERO_BURN_ERR otherwise.
+ **/
+
 BraseroBurnResult
 brasero_burn_session_can_blank (BraseroBurnSession *session)
 {
@@ -258,10 +289,6 @@ brasero_burn_session_can_blank (BraseroBurnSession *session)
 	return result;
 }
 
-/**
- *
- */
-
 static void
 brasero_caps_link_get_record_flags (BraseroCapsLink *link,
 				    BraseroMedia media,
@@ -602,6 +629,24 @@ brasero_caps_try_output_with_blanking (BraseroBurnCaps *self,
 				       io_flags);
 }
 
+/**
+ * brasero_burn_session_input_supported:
+ * @session: a #BraseroBurnSession
+ * @input: a #BraseroTrackType
+ * @use_flags: a #gboolean
+ *
+ * Given the various parameters stored in @session, this
+ * function checks whether a session with the data type
+ * @type could be burnt to the medium in the #BraseroDrive (set 
+ * through brasero_burn_session_set_burner ()).
+ * If @use_flags is TRUE, then flags are taken into account
+ * and are not if it is FALSE.
+ *
+ * Return value: a #BraseroBurnResult.
+ * BRASERO_BURN_OK if it is possible.
+ * BRASERO_BURN_ERR otherwise.
+ **/
+
 BraseroBurnResult
 brasero_burn_session_input_supported (BraseroBurnSession *session,
 				      BraseroTrackType *input,
@@ -661,6 +706,16 @@ brasero_burn_session_input_supported (BraseroBurnSession *session,
 	return BRASERO_BURN_OK;
 }
 
+/**
+ * brasero_burn_session_output_supported:
+ * @session: a #BraseroBurnSession *
+ * @output: a #BraseroTrackType *
+ *
+ * Make sure that the image type or medium type defined in @output can be
+ * created/burnt given the parameters and the current data set in @session.
+ *
+ * Return value: BRASERO_BURN_OK if the medium type or the image type can be used as an output.
+ **/
 BraseroBurnResult
 brasero_burn_session_output_supported (BraseroBurnSession *session,
 				       BraseroTrackType *output)
@@ -708,6 +763,13 @@ brasero_burn_session_output_supported (BraseroBurnSession *session,
 	return BRASERO_BURN_OK;
 }
 
+/* This function is not public API yet because it was too
+ * late; so use it internally for now. It's mainly for 
+ * convenience.*/
+gboolean
+brasero_drive_can_write_media (BraseroDrive *drive,
+                               BraseroMedia media);
+
 /**
  * This is only to be used in case one wants to copy using the same drive.
  * It determines the possible middle image type.
@@ -716,9 +778,11 @@ brasero_burn_session_output_supported (BraseroBurnSession *session,
 static BraseroBurnResult
 brasero_burn_caps_is_session_supported_same_src_dest (BraseroBurnCaps *self,
 						      BraseroBurnSession *session,
+                                                      BraseroTrackType *tmp_type,
 						      gboolean use_flags)
 {
 	GSList *iter;
+	BraseroDrive *burner;
 	BraseroTrackType input;
 	BraseroTrackType output;
 	BraseroImageFormat format;
@@ -749,6 +813,7 @@ brasero_burn_caps_is_session_supported_same_src_dest (BraseroBurnCaps *self,
 	/* Find one available output format */
 	format = BRASERO_IMAGE_FORMAT_CDRDAO;
 	output.type = BRASERO_TRACK_TYPE_IMAGE;
+	burner = brasero_burn_session_get_burner (session);
 
 	for (; format > BRASERO_IMAGE_FORMAT_NONE; format >>= 1) {
 		gboolean supported;
@@ -776,25 +841,68 @@ brasero_burn_caps_is_session_supported_same_src_dest (BraseroBurnCaps *self,
 			if (caps->type.type != BRASERO_TRACK_TYPE_DISC)
 				continue;
 
+			/* Make sure this is supported by the drive */
+			if (!brasero_drive_can_write_media (burner, caps->type.subtype.media))
+				continue;
+
 			result = brasero_caps_find_link (caps,
-							 TRUE,
+							 use_flags,
 							 session_flags,
 							 caps->type.subtype.media,
-							 &input,
+							 &output,
 							 BRASERO_PLUGIN_IO_ACCEPT_FILE);
 
 			BRASERO_BURN_LOG_DISC_TYPE (caps->type.subtype.media,
 						    "Tested medium (%s)",
 						    result ? "working":"not working");
 
-			if (result)
+			if (result) {
+				if (tmp_type) {
+					tmp_type->type = BRASERO_TRACK_TYPE_IMAGE;
+					tmp_type->subtype.img_format = format;
+				}
+					
 				return BRASERO_BURN_OK;
+			}
 		}
 	}
 
 	return BRASERO_BURN_NOT_SUPPORTED;
 }
 
+BraseroBurnResult
+brasero_burn_session_get_tmp_image_type_same_src_dest (BraseroBurnSession *session,
+                                                       BraseroTrackType *image_type)
+{
+	BraseroBurnResult result;
+	BraseroBurnCaps *self;
+
+	self = brasero_burn_caps_get_default ();
+	result = brasero_burn_caps_is_session_supported_same_src_dest (self,
+	                                                               session,
+	                                                               image_type,
+	                                                               TRUE);
+	g_object_unref (self);
+	return result;
+}
+
+/**
+ * brasero_burn_session_can_burn:
+ * @session: a #BraseroBurnSession
+ * @use_flags: a #gboolean
+ *
+ * Given the various parameters stored in @session, this
+ * function checks whether the data contained in @session
+ * can be burnt to the medium in the #BraseroDrive (set 
+ * through brasero_burn_session_set_burner ()).
+ * If @use_flags is set to TRUE the flags are taken into
+ * account, otherwise they are not.
+ *
+ * Return value: a #BraseroBurnResult.
+ * BRASERO_BURN_OK if it is possible.
+ * BRASERO_BURN_ERR otherwise.
+ **/
+
 BraseroBurnResult
 brasero_burn_session_can_burn (BraseroBurnSession *session,
 			       gboolean use_flags)
@@ -811,7 +919,7 @@ brasero_burn_session_can_burn (BraseroBurnSession *session,
 	if (brasero_burn_session_same_src_dest_drive (session)) {
 		BraseroBurnResult res;
 
-		res = brasero_burn_caps_is_session_supported_same_src_dest (self, session, use_flags);
+		res = brasero_burn_caps_is_session_supported_same_src_dest (self, session, NULL, use_flags);
 		g_object_unref (self);
 		return res;
 	}
@@ -872,6 +980,16 @@ brasero_burn_session_can_burn (BraseroBurnSession *session,
 	return BRASERO_BURN_OK;
 }
 
+/**
+ * brasero_burn_session_get_required_media_type:
+ * @session: a #BraseroBurnSession
+ *
+ * Return the medium types that could be used to burn
+ * @session.
+ *
+ * Return value: a #BraseroMedia
+ **/
+
 BraseroMedia
 brasero_burn_session_get_required_media_type (BraseroBurnSession *session)
 {
@@ -945,6 +1063,17 @@ brasero_burn_session_get_required_media_type (BraseroBurnSession *session)
 	return required_media;
 }
 
+/**
+ * brasero_burn_session_get_possible_output_formats:
+ * @session: a #BraseroBurnSession
+ * @formats: a #BraseroImageFormat
+ *
+ * Returns the disc image types that could be set to create
+ * an image given the current state of @session.
+ *
+ * Return value: a #guint. The number of formats available.
+ **/
+
 guint
 brasero_burn_session_get_possible_output_formats (BraseroBurnSession *session,
 						  BraseroImageFormat *formats)
@@ -978,6 +1107,16 @@ brasero_burn_session_get_possible_output_formats (BraseroBurnSession *session,
 	return num;
 }
 
+/**
+ * brasero_burn_session_get_default_output_format:
+ * @session: a #BraseroBurnSession
+ *
+ * Returns the default disc image type that should be set to create
+ * an image given the current state of @session.
+ *
+ * Return value: a #BraseroImageFormat
+ **/
+
 BraseroImageFormat
 brasero_burn_session_get_default_output_format (BraseroBurnSession *session)
 {
@@ -1318,7 +1457,7 @@ brasero_burn_caps_get_flags_for_medium (BraseroBurnCaps *self,
 	else if (session_flags & BRASERO_BURN_FLAG_BLANK_BEFORE_WRITE)
 		return BRASERO_BURN_NOT_SUPPORTED;
 
-	if (((*supported_flags) & BRASERO_BURN_FLAG_BLANK_BEFORE_WRITE)) {
+	if (((*supported_flags) & BRASERO_BURN_FLAG_BLANK_BEFORE_WRITE) != 0) {
 		BraseroBurnFlag blank_compulsory = BRASERO_BURN_FLAG_NONE;
 		BraseroBurnFlag blank_supported = BRASERO_BURN_FLAG_NONE;
 
@@ -1342,8 +1481,6 @@ brasero_burn_caps_get_flags_for_medium (BraseroBurnCaps *self,
 		 * Ex: a CDRW with data appendable can be either appended (then
 		 * no DAO possible) or blanked and written (DAO possible). */
 
-		(*supported_flags) |= BRASERO_BURN_FLAG_BLANK_BEFORE_WRITE;
-
 		/* result here is the result of the first operation, so if it
 		 * failed, BLANK before becomes compulsory. */
 		if (result != BRASERO_BURN_OK)
@@ -1501,6 +1638,23 @@ brasero_burn_caps_get_flags_same_src_dest (BraseroBurnCaps *self,
 	return BRASERO_BURN_OK;
 }
 
+/**
+ * brasero_burn_session_get_burn_flags:
+ * @session: a #BraseroBurnSession
+ * @supported: a #BraseroBurnFlag or NULL
+ * @compulsory: a #BraseroBurnFlag or NULL
+ *
+ * Given the various parameters stored in @session, this function
+ * stores:
+ * - the flags that can be used (@supported)
+ * - the flags that must be used (@compulsory)
+ * when writing @session to a disc.
+ *
+ * Return value: a #BraseroBurnResult.
+ * BRASERO_BURN_OK if the retrieval was successful.
+ * BRASERO_BURN_ERR otherwise.
+ **/
+
 BraseroBurnResult
 brasero_burn_session_get_burn_flags (BraseroBurnSession *session,
 				     BraseroBurnFlag *supported,
@@ -1546,6 +1700,7 @@ brasero_burn_session_get_burn_flags (BraseroBurnSession *session,
 
 	/* special case */
 	if (brasero_burn_session_same_src_dest_drive (session)) {
+		BRASERO_BURN_LOG ("Same source and destination");
 		result = brasero_burn_caps_get_flags_same_src_dest (self,
 								    session,
 								    &supported_flags,
