@@ -48,6 +48,7 @@
 
 #include "burn-basics.h"
 #include "burn-debug.h"
+#include "libbrasero-marshal.h"
 #include "burn-image-format.h"
 
 #include "brasero-medium.h"
@@ -107,7 +108,10 @@ typedef struct _BraseroBurnSessionPrivate BraseroBurnSessionPrivate;
 #define BRASERO_STR_EQUAL(a, b)	((!(a) && !(b)) || ((a) && (b) && !strcmp ((a), (b))))
 
 typedef enum {
-	INPUT_CHANGED_SIGNAL,
+	FLAGS_CHANGED_SIGNAL,
+	TRACK_ADDED_SIGNAL,
+	TRACK_REMOVED_SIGNAL,
+	TRACK_CHANGED_SIGNAL,
 	OUTPUT_CHANGED_SIGNAL,
 	LAST_SIGNAL
 } BraseroBurnSessionSignalType;
@@ -136,7 +140,7 @@ brasero_session_settings_clean (BraseroSessionSetting *settings)
 	memset (settings, 0, sizeof (BraseroSessionSetting));
 }
 
-void
+static void
 brasero_session_settings_copy (BraseroSessionSetting *dest,
 			       BraseroSessionSetting *original)
 {
@@ -159,12 +163,13 @@ brasero_session_settings_free (BraseroSessionSetting *settings)
 }
 
 static void
-brasero_burn_session_track_changed (BraseroDrive *drive,
+brasero_burn_session_track_changed (BraseroTrack *track,
 				    BraseroBurnSession *self)
 {
 	g_signal_emit (self,
-		       brasero_burn_session_signals [INPUT_CHANGED_SIGNAL],
-		       0);
+		       brasero_burn_session_signals [TRACK_CHANGED_SIGNAL],
+		       0,
+		       track);
 }
 
 static void
@@ -202,6 +207,8 @@ static void
 brasero_burn_session_free_tracks (BraseroBurnSession *self)
 {
 	BraseroBurnSessionPrivate *priv;
+	GSList *iter;
+	GSList *next;
 
 	g_return_if_fail (BRASERO_IS_BURN_SESSION (self));
 
@@ -209,18 +216,40 @@ brasero_burn_session_free_tracks (BraseroBurnSession *self)
 
 	brasero_burn_session_stop_tracks_monitoring (self);
 
-	g_slist_foreach (priv->tracks, (GFunc) g_object_unref, NULL);
-	g_slist_free (priv->tracks);
-	priv->tracks = NULL;
+	for (iter = priv->tracks; iter; iter = next) {
+		BraseroTrack *track;
 
-	g_signal_emit (self,
-		       brasero_burn_session_signals [INPUT_CHANGED_SIGNAL],
-		       0);
+		track = iter->data;
+		next = iter->next;
+		priv->tracks = g_slist_remove (priv->tracks, track);
+		g_signal_emit (self,
+			       brasero_burn_session_signals [TRACK_REMOVED_SIGNAL],
+			       0,
+			       track,
+			       0);
+		g_object_unref (track);
+	}
 }
 
+
+/**
+ * brasero_burn_session_add_track:
+ * @session: a #BraseroBurnSession
+ * @new_track: a #BraseroTrack or NULL
+ * @sibling: a #BraseroTrack or NULL
+ *
+ * Inserts a new track after @sibling or appended if @sibling is NULL. If @track is NULL then all tracks
+ * already in @session will be removed.
+ * NOTE: if there are already #BraseroTrack objects inserted in the session and if they are not
+ * of the same type as @new_track then they will be removed before the insertion of @new_track.
+ *
+ * Return value: a #BraseroBurnResult. BRASERO_BURN_OK if the track was successfully inserted or BRASERO_BURN_ERR otherwise.
+ **/
+
 BraseroBurnResult
 brasero_burn_session_add_track (BraseroBurnSession *self,
-				BraseroTrack *new_track)
+				BraseroTrack *new_track,
+				BraseroTrack *sibling)
 {
 	BraseroBurnSessionPrivate *priv;
 
@@ -228,6 +257,12 @@ brasero_burn_session_add_track (BraseroBurnSession *self,
 
 	priv = BRASERO_BURN_SESSION_PRIVATE (self);
 
+	/* Prevent adding the same tracks several times */
+	if (g_slist_find (priv->tracks, new_track)) {
+		BRASERO_BURN_LOG ("Tried to add the same track multiple times");
+		return BRASERO_BURN_OK;
+	}
+
 	if (!new_track) {
 		if (!priv->tracks)
 			return BRASERO_BURN_OK;
@@ -239,43 +274,144 @@ brasero_burn_session_add_track (BraseroBurnSession *self,
 	g_object_ref (new_track);
 	if (!priv->tracks) {
 		/* we only need to emit the signal here since if there are
-		 * multiple tracks they must be exactly of the same time */
+		 * multiple tracks they must be exactly of the same type */
 		priv->tracks = g_slist_prepend (NULL, new_track);
 		brasero_burn_session_start_track_monitoring (self, new_track);
 
 		/* if (!brasero_track_type_equal (priv->input, &new_type)) */
 		g_signal_emit (self,
-			       brasero_burn_session_signals [INPUT_CHANGED_SIGNAL],
-			       0);
+			       brasero_burn_session_signals [TRACK_ADDED_SIGNAL],
+			       0,
+			       new_track);
 
 		return BRASERO_BURN_OK;
 	}
 
 	/* if there is already a track, then we replace it on condition that it
 	 * has the same type and it's not AUDIO (only one allowed to have many) */
-	if (!BRASERO_IS_TRACK_STREAM (new_track)) {
-		brasero_burn_session_stop_tracks_monitoring (self);
+	if (!BRASERO_IS_TRACK_STREAM (new_track)
+	||  !BRASERO_IS_TRACK_STREAM (priv->tracks->data))
+		brasero_burn_session_free_tracks (self);
 
-		g_slist_foreach (priv->tracks, (GFunc) g_object_unref, NULL);
-		g_slist_free (priv->tracks);
+	brasero_burn_session_start_track_monitoring (self, new_track);
+	if (sibling) {
+		GSList *sibling_node;
 
-		priv->tracks = g_slist_prepend (NULL, new_track);
-		brasero_burn_session_start_track_monitoring (self, new_track);
+		sibling_node = g_slist_find (priv->tracks, sibling);
+		priv->tracks = g_slist_insert_before (priv->tracks,
+						      sibling_node,
+						      new_track);
 	}
-	else {
-		brasero_burn_session_start_track_monitoring (self, new_track);
+	else
 		priv->tracks = g_slist_append (priv->tracks, new_track);
-	}
 
-	/* Always emit the signal even when adding another BraseroTrackStream
-	 * since the size has probably changed. */
 	g_signal_emit (self,
-		       brasero_burn_session_signals [INPUT_CHANGED_SIGNAL],
-		       0);
+		       brasero_burn_session_signals [TRACK_ADDED_SIGNAL],
+		       0,
+		       new_track);
 
 	return BRASERO_BURN_OK;
 }
 
+/**
+ * brasero_burn_session_move_track:
+ * @session: a #BraseroBurnSession
+ * @track: a #BraseroTrack
+ * @sibling: a #BraseroTrack or NULL
+ *
+ * Moves @track after @sibling; if @sibling is NULL then it is appended.
+ *
+ * Return value: a #BraseroBurnResult. BRASERO_BURN_OK if the track was successfully moved or BRASERO_BURN_ERR otherwise.
+ **/
+
+BraseroBurnResult
+brasero_burn_session_move_track (BraseroBurnSession *session,
+				 BraseroTrack *track,
+				 BraseroTrack *sibling)
+{
+	BraseroBurnSessionPrivate *priv;
+	guint former_position;
+
+	g_return_val_if_fail (BRASERO_IS_BURN_SESSION (session), BRASERO_BURN_ERR);
+
+	priv = BRASERO_BURN_SESSION_PRIVATE (session);
+
+	/* Find the track, remove it */
+	former_position = g_slist_index (priv->tracks, track);
+	priv->tracks = g_slist_remove (priv->tracks, track);
+	g_signal_emit (session,
+		       brasero_burn_session_signals [TRACK_REMOVED_SIGNAL],
+		       0,
+		       track,
+		       former_position);
+
+	/* Re-add it */
+	if (sibling) {
+		GSList *sibling_node;
+
+		sibling_node = g_slist_find (priv->tracks, sibling);
+		priv->tracks = g_slist_insert_before (priv->tracks,
+						      sibling_node,
+						      track);
+	}
+	else
+		priv->tracks = g_slist_append (priv->tracks, track);
+
+	g_signal_emit (session,
+		       brasero_burn_session_signals [TRACK_ADDED_SIGNAL],
+		       0,
+		       track);
+
+	return BRASERO_BURN_OK;
+}
+
+/**
+ * brasero_burn_session_remove_track:
+ * @session: a #BraseroBurnSession
+ * @track: a #BraseroTrack
+ *
+ * Removes @track from @session.
+ *
+ * Return value: a #BraseroBurnResult. BRASERO_BURN_OK if the track was successfully removed or BRASERO_BURN_ERR otherwise.
+ **/
+
+BraseroBurnResult
+brasero_burn_session_remove_track (BraseroBurnSession *session,
+				   BraseroTrack *track)
+{
+	BraseroBurnSessionPrivate *priv;
+	guint former_position;
+
+	g_return_val_if_fail (BRASERO_IS_BURN_SESSION (session), BRASERO_BURN_ERR);
+
+	priv = BRASERO_BURN_SESSION_PRIVATE (session);
+
+	/* Find the track, remove it */
+	former_position = g_slist_index (priv->tracks, track);
+	priv->tracks = g_slist_remove (priv->tracks, track);
+	g_signal_handlers_disconnect_by_func (track,
+					      brasero_burn_session_track_changed,
+					      session);
+
+	g_signal_emit (session,
+		       brasero_burn_session_signals [TRACK_REMOVED_SIGNAL],
+		       0,
+		       track,
+		       former_position);
+
+	g_object_unref (track);
+	return BRASERO_BURN_OK;
+}
+
+/**
+ * brasero_burn_session_get_tracks:
+ * @session: a #BraseroBurnSession
+ *
+ * Returns the list of #BraseroTrack added to @session.
+ *
+ * Return value: a #GSList or #BraseroTrack object. Do not unref the objects in the list nor destroy the list.
+ **/
+
 GSList *
 brasero_burn_session_get_tracks (BraseroBurnSession *self)
 {
@@ -288,6 +424,19 @@ brasero_burn_session_get_tracks (BraseroBurnSession *self)
 	return priv->tracks;
 }
 
+/**
+ * brasero_burn_session_get_status:
+ * @session: a #BraseroBurnSession
+ * @status: a #BraseroTrackStatus
+ *
+ * Sets @status to reflect whether @session is ready to be used.
+ *
+ * Return value: a #BraseroBurnResult.
+ * BRASERO_BURN_OK if it was successful
+ * BRASERO_BURN_NOT_READY if @track needs more time for processing
+ * BRASERO_BURN_ERR if something is wrong or if it is empty
+ **/
+
 BraseroBurnResult
 brasero_burn_session_get_status (BraseroBurnSession *session,
 				 BraseroStatus *status)
@@ -344,6 +493,20 @@ brasero_burn_session_get_status (BraseroBurnSession *session,
 	return BRASERO_BURN_OK;
 }
 
+/**
+ * brasero_burn_session_get_size:
+ * @session: a #BraseroBurnSession
+ * @blocks: a #goffset or NULL
+ * @bytes: a #goffset or NULL
+ *
+ * Returns the size of the data contained by @session in bytes or in sectors
+ *
+ * Return value: a #BraseroBurnResult.
+ * BRASERO_BURN_OK if it was successful
+ * BRASERO_BURN_NOT_READY if @track needs more time for processing the size
+ * BRASERO_BURN_ERR if something is wrong or if it is empty
+ **/
+
 BraseroBurnResult
 brasero_burn_session_get_size (BraseroBurnSession *session,
 			       goffset *blocks,
@@ -389,6 +552,17 @@ brasero_burn_session_get_size (BraseroBurnSession *session,
 	return BRASERO_BURN_OK;
 }
 
+/**
+ * brasero_burn_session_get_input_type:
+ * @session: a #BraseroBurnSession
+ * @type: a #BraseroTrackType or NULL
+ *
+ * Sets @type to reflect the type of data contained in @session
+ *
+ * Return value: a #BraseroBurnResult.
+ * BRASERO_BURN_OK if it was successful
+ **/
+
 BraseroBurnResult
 brasero_burn_session_get_input_type (BraseroBurnSession *self,
 				     BraseroTrackType *type)
@@ -411,10 +585,6 @@ brasero_burn_session_get_input_type (BraseroBurnSession *self,
 	return BRASERO_BURN_OK;
 }
 
-/**
- *
- */
-
 static void
 brasero_burn_session_dest_media_added (BraseroDrive *drive,
 				       BraseroMedium *medium,
@@ -438,6 +608,15 @@ brasero_burn_session_dest_media_removed (BraseroDrive *drive,
 		       medium);
 }
 
+/**
+ * brasero_burn_session_set_burner:
+ * @session: a #BraseroBurnSession
+ * @drive: a #BraseroDrive
+ *
+ * Sets the #BraseroDrive that should be used to burn the session contents.
+ *
+ **/
+
 void
 brasero_burn_session_set_burner (BraseroBurnSession *self,
 				 BraseroDrive *drive)
@@ -454,7 +633,7 @@ brasero_burn_session_set_burner (BraseroBurnSession *self,
 
 	former = brasero_drive_get_medium (priv->settings->burner);
 	if (former)
-		former = g_object_ref (former);
+		g_object_ref (former);
 
 	/* If there was no drive before no need for a changing signal */
 	if (priv->settings->burner) {
@@ -491,10 +670,20 @@ brasero_burn_session_set_burner (BraseroBurnSession *self,
 		       brasero_burn_session_signals [OUTPUT_CHANGED_SIGNAL],
 		       0,
 		       former);
+
 	if (former)
 		g_object_unref (former);
 }
 
+/**
+ * brasero_burn_session_get_burner:
+ * @session: a #BraseroBurnSession
+ *
+ * Returns the #BraseroDrive that should be used to burn the session contents.
+ *
+ * Return value: a #BraseroDrive or NULL. Do not unref after use.
+ **/
+
 BraseroDrive *
 brasero_burn_session_get_burner (BraseroBurnSession *self)
 {
@@ -506,8 +695,22 @@ brasero_burn_session_get_burner (BraseroBurnSession *self)
 	return priv->settings->burner;
 }
 
+/**
+ * brasero_burn_session_set_rate:
+ * @session: a #BraseroBurnSession
+ * @rate: a #guint64
+ *
+ * Sets the speed at which the medium should be burnt.
+ * NOTE: before using this function a #BraseroDrive should have been
+ * set with brasero_burn_session_set_burner ().
+ *
+ * Return value: a #BraseroBurnResult.
+ * BRASERO_BURN_OK if it was successful; BRASERO_BURN_ERR otherwise.
+ **/
+
 BraseroBurnResult
-brasero_burn_session_set_rate (BraseroBurnSession *self, guint64 rate)
+brasero_burn_session_set_rate (BraseroBurnSession *self,
+                               guint64 rate)
 {
 	BraseroBurnSessionPrivate *priv;
 
@@ -522,6 +725,17 @@ brasero_burn_session_set_rate (BraseroBurnSession *self, guint64 rate)
 	return BRASERO_BURN_OK;
 }
 
+/**
+ * brasero_burn_session_get_rate:
+ * @session: a #BraseroBurnSession
+ *
+ * Returns the speed at which the medium should be burnt.
+ * NOTE: before using this function a #BraseroDrive should have been
+ * set with brasero_burn_session_set_burner ().
+ *
+ * Return value: a #guint64 or 0.
+ **/
+
 guint64
 brasero_burn_session_get_rate (BraseroBurnSession *self)
 {
@@ -545,32 +759,58 @@ brasero_burn_session_get_rate (BraseroBurnSession *self)
 }
 
 /**
- * This function returns a path only if we should output to a file image
- * and not burn.
- */
+ * brasero_burn_session_get_output:
+ * @session: a #BraseroBurnSession
+ * @image: a #gchar or NULL
+ * @toc: a #gchar or NULL
+ *
+ * When the contents of @session should be written to a
+ * file then this function returns the image path (and if
+ * necessary a toc path).
+ * @image and @toc should be freed if not used anymore.
+ *
+ * NOTE: before using this function a #BraseroDrive should have been
+ * set with brasero_burn_session_set_burner () and it should be the
+ * fake drive (see brasero_drive_is_fake ()).
+ *
+ * Return value: a #BraseroBurnResult.
+ * BRASERO_BURN_OK if it was successful; BRASERO_BURN_ERR otherwise.
+ **/
 
 BraseroBurnResult
 brasero_burn_session_get_output (BraseroBurnSession *self,
-				 gchar **image_ret,
-				 gchar **toc_ret,
-				 GError **error)
+				 gchar **image,
+				 gchar **toc)
 {
-	gchar *toc = NULL;
-	gchar *image = NULL;
+	BraseroBurnSessionClass *klass;
 	BraseroBurnSessionPrivate *priv;
 
-	g_return_val_if_fail (BRASERO_IS_BURN_SESSION (self), BRASERO_BURN_ERR);
+	g_return_val_if_fail (BRASERO_IS_BURN_SESSION (self), BRASERO_IMAGE_FORMAT_NONE);
 
 	priv = BRASERO_BURN_SESSION_PRIVATE (self);
-
 	if (!BRASERO_BURN_SESSION_WRITE_TO_FILE (priv)) {
 		BRASERO_BURN_LOG ("no file disc");
 		return BRASERO_BURN_ERR;
 	}
 
+	klass = BRASERO_BURN_SESSION_GET_CLASS (self);
+	return klass->get_output_path (self, image, toc);
+}
+
+
+static BraseroBurnResult
+brasero_burn_session_get_output_path_real (BraseroBurnSession *self,
+					   gchar **image_ret,
+					   gchar **toc_ret)
+{
+	gchar *toc = NULL;
+	gchar *image = NULL;
+	BraseroBurnSessionPrivate *priv;
+
+	priv = BRASERO_BURN_SESSION_PRIVATE (self);
+
 	image = g_strdup (priv->settings->image);
 	toc = g_strdup (priv->settings->toc);
-
 	if (!image && !toc)
 		return BRASERO_BURN_ERR;
 
@@ -585,12 +825,6 @@ brasero_burn_session_get_output (BraseroBurnSession *self,
 			complement = brasero_image_format_get_complement (format, toc);
 			if (!complement) {
 				BRASERO_BURN_LOG ("no output specified");
-
-				g_set_error (error,
-					     BRASERO_BURN_ERROR,
-					     BRASERO_BURN_ERROR_OUTPUT_NONE,
-					     _("No path was specified for the image output"));
-
 				g_free (toc);
 				return BRASERO_BURN_ERR;
 			}
@@ -601,11 +835,6 @@ brasero_burn_session_get_output (BraseroBurnSession *self,
 			*image_ret = image;
 		else {
 			BRASERO_BURN_LOG ("no output specified");
-
-			g_set_error (error,
-				     BRASERO_BURN_ERROR,
-				     BRASERO_BURN_ERROR_OUTPUT_NONE,
-				     _("No path was specified for the image output"));
 			return BRASERO_BURN_ERR;
 		}
 	}
@@ -620,18 +849,42 @@ brasero_burn_session_get_output (BraseroBurnSession *self,
 	return BRASERO_BURN_OK;
 }
 
+/**
+ * brasero_burn_session_get_output_format:
+ * @session: a #BraseroBurnSession
+ *
+ * When the contents of @session should be written to a
+ * file then this function returns the format of the image to write.
+ *
+ * NOTE: before using this function a #BraseroDrive should have been
+ * set with brasero_burn_session_set_burner () and it should be the
+ * fake drive (see brasero_drive_is_fake ()).
+ *
+ * Return value: a #BraseroImageFormat. The format of the image to be written.
+ **/
+
 BraseroImageFormat
 brasero_burn_session_get_output_format (BraseroBurnSession *self)
 {
+	BraseroBurnSessionClass *klass;
 	BraseroBurnSessionPrivate *priv;
 
 	g_return_val_if_fail (BRASERO_IS_BURN_SESSION (self), BRASERO_IMAGE_FORMAT_NONE);
 
 	priv = BRASERO_BURN_SESSION_PRIVATE (self);
-
 	if (!BRASERO_BURN_SESSION_WRITE_TO_FILE (priv))
 		return BRASERO_IMAGE_FORMAT_NONE;
 
+	klass = BRASERO_BURN_SESSION_GET_CLASS (self);
+	return klass->get_output_format (self);
+}
+
+static BraseroImageFormat
+brasero_burn_session_get_output_format_real (BraseroBurnSession *self)
+{
+	BraseroBurnSessionPrivate *priv;
+
+	priv = BRASERO_BURN_SESSION_PRIVATE (self);
 	return priv->settings->format;
 }
 
@@ -669,16 +922,14 @@ brasero_burn_session_set_image_output_real (BraseroBurnSession *self,
 	priv->settings->format = format;
 }
 
-BraseroBurnResult
-brasero_burn_session_set_image_output_full (BraseroBurnSession *self,
+static BraseroBurnResult
+brasero_burn_session_set_output_image_real (BraseroBurnSession *self,
 					    BraseroImageFormat format,
 					    const gchar *image,
 					    const gchar *toc)
 {
 	BraseroBurnSessionPrivate *priv;
 
-	g_return_val_if_fail (BRASERO_IS_BURN_SESSION (self), BRASERO_BURN_ERR);
-
 	priv = BRASERO_BURN_SESSION_PRIVATE (self);
 
 	if (priv->settings->format == format
@@ -710,7 +961,7 @@ brasero_burn_session_set_image_output_full (BraseroBurnSession *self,
 		brasero_burn_session_set_image_output_real (self, format, image, toc);
 
 		monitor = brasero_medium_monitor_get_default ();
-		list = brasero_medium_monitor_get_media (monitor,BRASERO_MEDIA_TYPE_FILE);
+		list = brasero_medium_monitor_get_media (monitor, BRASERO_MEDIA_TYPE_FILE);
 		drive = brasero_medium_get_drive (list->data);
 		brasero_burn_session_set_burner (self, drive);
 		g_object_unref (monitor);
@@ -728,8 +979,49 @@ brasero_burn_session_set_image_output_full (BraseroBurnSession *self,
 }
 
 /**
+ * brasero_burn_session_set_image_output_full:
+ * @session: a #BraseroBurnSession
+ * @format: a #BraseroImageFormat
+ * @image: a #gchar or NULL
+ * @toc: a #gchar or NULL
  *
- */
+ * When the contents of @session should be written to a
+ * file, this function sets the different parameters of this
+ * image like its path (and the one of the associated toc if
+ * necessary) and its format.
+ *
+ * NOTE: after a call to this function the #BraseroDrive for
+ * @session will be the fake #BraseroDrive.
+ *
+ * Return value: a #BraseroBurnResult. BRASERO_BURN_OK if it was successfully set;
+ * BRASERO_BURN_ERR otherwise.
+ **/
+
+BraseroBurnResult
+brasero_burn_session_set_image_output_full (BraseroBurnSession *self,
+					    BraseroImageFormat format,
+					    const gchar *image,
+					    const gchar *toc)
+{
+	BraseroBurnSessionClass *klass;
+
+	g_return_val_if_fail (BRASERO_IS_BURN_SESSION (self), BRASERO_BURN_ERR);
+
+	klass = BRASERO_BURN_SESSION_GET_CLASS (self);
+	return klass->set_output_image (self, format, image, toc);
+}
+
+/**
+ * brasero_burn_session_set_tmpdir:
+ * @session: a #BraseroBurnSession
+ * @path: a #gchar or NULL
+ *
+ * Sets the path of the directory in which to write temporary directories and files.
+ * If set to NULL then the result of g_get_tmp_dir () will be used.
+ *
+ * Return value: a #BraseroBurnResult. BRASERO_BURN_OK if it was successfully set;
+ * BRASERO_BURN_ERR otherwise.
+ **/
 
 BraseroBurnResult
 brasero_burn_session_set_tmpdir (BraseroBurnSession *self,
@@ -756,6 +1048,15 @@ brasero_burn_session_set_tmpdir (BraseroBurnSession *self,
 	return BRASERO_BURN_OK;
 }
 
+/**
+ * brasero_burn_session_get_tmpdir:
+ * @session: a #BraseroBurnSession
+ *
+ * Returns the path of the directory in which to write temporary directories and files.
+ *
+ * Return value: a #gchar. The path to the temporary directory.
+ **/
+
 const gchar *
 brasero_burn_session_get_tmpdir (BraseroBurnSession *self)
 {
@@ -767,6 +1068,21 @@ brasero_burn_session_get_tmpdir (BraseroBurnSession *self)
 	return priv->settings->tmpdir? priv->settings->tmpdir:g_get_tmp_dir ();
 }
 
+/**
+ * brasero_burn_session_get_tmp_dir:
+ * @session: a #BraseroBurnSession
+ * @path: a #gchar or NULL
+ * @error: a #GError
+ *
+ * Creates then returns (in @path) a temporary directory at the proper location.
+ * On error, @error is set appropriately.
+ * See brasero_burn_session_set_tmpdir ().
+ * This function is used internally and is not public API.
+ *
+ * Return value: a #BraseroBurnResult. BRASERO_BURN_OK if it was successfully set;
+ * BRASERO_BURN_ERR otherwise.
+ **/
+
 BraseroBurnResult
 brasero_burn_session_get_tmp_dir (BraseroBurnSession *self,
 				  gchar **path,
@@ -816,6 +1132,23 @@ brasero_burn_session_get_tmp_dir (BraseroBurnSession *self,
 	return BRASERO_BURN_OK;
 }
 
+/**
+ * brasero_burn_session_get_tmp_file:
+ * @session: a #BraseroBurnSession
+ * @suffix: a #gchar
+ * @path: a #gchar or NULL
+ * @error: a #GError
+ *
+ * Creates then returns (in @path) a temporary file at the proper location. Its name
+ * will be appended with suffix.
+ * On error, @error is set appropriately.
+ * See brasero_burn_session_set_tmpdir ().
+ * This function is used internally and is not public API.
+ *
+ * Return value: a #BraseroBurnResult. BRASERO_BURN_OK if it was successfully set;
+ * BRASERO_BURN_ERR otherwise.
+ **/
+
 BraseroBurnResult
 brasero_burn_session_get_tmp_file (BraseroBurnSession *self,
 				   const gchar *suffix,
@@ -912,6 +1245,10 @@ brasero_burn_session_get_image_complement (BraseroBurnSession *self,
 	return retval;
 }
 
+/**
+ * This function is used internally and is not public API
+ */
+
 BraseroBurnResult
 brasero_burn_session_get_tmp_image (BraseroBurnSession *self,
 				    BraseroImageFormat format,
@@ -969,6 +1306,15 @@ brasero_burn_session_get_tmp_image (BraseroBurnSession *self,
  * used to modify session flags.
  */
 
+/**
+ * brasero_burn_session_set_flags:
+ * @session: a #BraseroBurnSession
+ * @flags: a #BraseroBurnFlag
+ *
+ * Replaces the current flags set in @session with @flags.
+ *
+ **/
+
 void
 brasero_burn_session_set_flags (BraseroBurnSession *self,
 			        BraseroBurnFlag flags)
@@ -978,9 +1324,24 @@ brasero_burn_session_set_flags (BraseroBurnSession *self,
 	g_return_if_fail (BRASERO_IS_BURN_SESSION (self));
 
 	priv = BRASERO_BURN_SESSION_PRIVATE (self);
+	if (priv->settings->flags == flags)
+		return;
+
 	priv->settings->flags = flags;
+	g_signal_emit (self,
+		       brasero_burn_session_signals [FLAGS_CHANGED_SIGNAL],
+		       0);
 }
 
+/**
+ * brasero_burn_session_add_flag:
+ * @session: a #BraseroBurnSession
+ * @flags: a #BraseroBurnFlag
+ *
+ * Merges the current flags set in @session with @flags.
+ *
+ **/
+
 void
 brasero_burn_session_add_flag (BraseroBurnSession *self,
 			       BraseroBurnFlag flag)
@@ -990,9 +1351,24 @@ brasero_burn_session_add_flag (BraseroBurnSession *self,
 	g_return_if_fail (BRASERO_IS_BURN_SESSION (self));
 
 	priv = BRASERO_BURN_SESSION_PRIVATE (self);
+	if ((priv->settings->flags & flag) == flag)
+		return;
+
 	priv->settings->flags |= flag;
+	g_signal_emit (self,
+		       brasero_burn_session_signals [FLAGS_CHANGED_SIGNAL],
+		       0);
 }
 
+/**
+ * brasero_burn_session_remove_flag:
+ * @session: a #BraseroBurnSession
+ * @flags: a #BraseroBurnFlag
+ *
+ * Removes @flags from the current flags set for @session.
+ *
+ **/
+
 void
 brasero_burn_session_remove_flag (BraseroBurnSession *self,
 				  BraseroBurnFlag flag)
@@ -1002,9 +1378,24 @@ brasero_burn_session_remove_flag (BraseroBurnSession *self,
 	g_return_if_fail (BRASERO_IS_BURN_SESSION (self));
 
 	priv = BRASERO_BURN_SESSION_PRIVATE (self);
+	if ((priv->settings->flags & flag) == 0)
+		return;
+
 	priv->settings->flags &= ~flag;
+	g_signal_emit (self,
+		       brasero_burn_session_signals [FLAGS_CHANGED_SIGNAL],
+		       0);
 }
 
+/**
+ * brasero_burn_session_get_flags:
+ * @session: a #BraseroBurnSession
+ *
+ * Returns the current flags set for @session.
+ *
+ * Return value: a #BraseroBurnFlag.
+ **/
+
 BraseroBurnFlag
 brasero_burn_session_get_flags (BraseroBurnSession *self)
 {
@@ -1019,7 +1410,16 @@ brasero_burn_session_get_flags (BraseroBurnSession *self)
 /**
  * Used to set the label or the title of an album. 
  */
- 
+
+/**
+ * brasero_burn_session_set_label:
+ * @session: a #BraseroBurnSession
+ * @label: a #gchar
+ *
+ * Sets the label for @session.
+ *
+ **/
+
 void
 brasero_burn_session_set_label (BraseroBurnSession *self,
 				const gchar *label)
@@ -1057,6 +1457,15 @@ brasero_burn_session_set_label (BraseroBurnSession *self,
 	}
 }
 
+/**
+ * brasero_burn_session_get_label:
+ * @session: a #BraseroBurnSession
+ *
+ * Returns the label (a string) set for @session.
+ *
+ * Return value: a #gchar or NULL. Do not free after use.
+ **/
+
 const gchar *
 brasero_burn_session_get_label (BraseroBurnSession *self)
 {
@@ -1077,6 +1486,19 @@ brasero_burn_session_tag_value_free (gpointer user_data)
 	g_free (value);
 }
 
+/**
+ * brasero_burn_session_tag_remove:
+ * @session: a #BraseroBurnSession
+ * @tag: a #gchar *
+ *
+ * Removes a value associated with @session through
+ * brasero_session_tag_add ().
+ *
+ * Return value: a #BraseroBurnResult.
+ * BRASERO_BURN_OK if the retrieval was successful
+ * BRASERO_BURN_ERR otherwise
+ **/
+
 BraseroBurnResult
 brasero_burn_session_tag_remove (BraseroBurnSession *self,
 				 const gchar *tag)
@@ -1093,6 +1515,22 @@ brasero_burn_session_tag_remove (BraseroBurnSession *self,
 	return BRASERO_BURN_OK;
 }
 
+/**
+ * brasero_burn_session_tag_add:
+ * @session: a #BraseroBurnSession
+ * @tag: a #gchar *
+ * @value: a #GValue *
+ *
+ * Associates a new @tag with @session. This can be used
+ * to pass arbitrary information for plugins, like parameters
+ * for video discs, ...
+ * See brasero-tags.h for a list of knowns tags.
+ *
+ * Return value: a #BraseroBurnResult.
+ * BRASERO_BURN_OK if it was successful,
+ * BRASERO_BURN_ERR otherwise.
+ **/
+
 BraseroBurnResult
 brasero_burn_session_tag_add (BraseroBurnSession *self,
 			      const gchar *tag,
@@ -1112,6 +1550,21 @@ brasero_burn_session_tag_add (BraseroBurnSession *self,
 	return BRASERO_BURN_OK;
 }
 
+/**
+ * brasero_burn_session_tag_lookup:
+ * @session: a #BraseroBurnSession
+ * @tag: a #gchar *
+ * @value: a #GValue
+ *
+ * Retrieves a value associated with @session through
+ * brasero_session_tag_add () and stores it in @value. Do
+ * not destroy @value afterwards as it is not a copy.
+ *
+ * Return value: a #BraseroBurnResult.
+ * BRASERO_BURN_OK if the retrieval was successful
+ * BRASERO_BURN_ERR otherwise
+ **/
+
 BraseroBurnResult
 brasero_burn_session_tag_lookup (BraseroBurnSession *self,
 				 const gchar *tag,
@@ -1194,7 +1647,6 @@ brasero_burn_session_pop_settings (BraseroBurnSession *self)
 	brasero_session_settings_copy (priv->settings, settings);
 
 	brasero_session_settings_free (settings);
-
 	if (priv->settings->burner) {
 		priv->dest_added_sig = g_signal_connect (priv->settings->burner,
 							 "medium-added",
@@ -1210,6 +1662,7 @@ brasero_burn_session_pop_settings (BraseroBurnSession *self)
 		       brasero_burn_session_signals [OUTPUT_CHANGED_SIGNAL],
 		       0,
 		       former);
+
 	if (former)
 		g_object_unref (former);
 }
@@ -1218,6 +1671,7 @@ void
 brasero_burn_session_push_tracks (BraseroBurnSession *self)
 {
 	BraseroBurnSessionPrivate *priv;
+	GSList *iter;
 
 	g_return_if_fail (BRASERO_IS_BURN_SESSION (self));
 
@@ -1226,41 +1680,37 @@ brasero_burn_session_push_tracks (BraseroBurnSession *self)
 	brasero_burn_session_stop_tracks_monitoring (self);
 
 	priv->pile_tracks = g_slist_prepend (priv->pile_tracks, priv->tracks);
+	iter = priv->tracks;
 	priv->tracks = NULL;
 
-	g_signal_emit (self,
-		       brasero_burn_session_signals [INPUT_CHANGED_SIGNAL],
-		       0);
+	for (; iter; iter = iter->next) {
+		BraseroTrack *track;
+
+		track = iter->data;
+		g_signal_emit (self,
+			       brasero_burn_session_signals [TRACK_REMOVED_SIGNAL],
+			       0,
+			       track,
+			       0);
+	}
 }
 
-void
+BraseroBurnResult
 brasero_burn_session_pop_tracks (BraseroBurnSession *self)
 {
 	GSList *sources;
 	BraseroBurnSessionPrivate *priv;
 
-	g_return_if_fail (BRASERO_IS_BURN_SESSION (self));
+	g_return_val_if_fail (BRASERO_IS_BURN_SESSION (self), FALSE);
 
 	priv = BRASERO_BURN_SESSION_PRIVATE (self);
 
 	/* Don't go further if there is no list of tracks on the pile */
 	if (!priv->pile_tracks)
-		return;
+		return BRASERO_BURN_OK;
 
-	if (priv->tracks) {
-		brasero_burn_session_stop_tracks_monitoring (self);
-
-		g_slist_foreach (priv->tracks, (GFunc) g_object_unref, NULL);
-		g_slist_free (priv->tracks);
-		priv->tracks = NULL;
-
-		if (!priv->pile_tracks) {
-			g_signal_emit (self,
-				       brasero_burn_session_signals [INPUT_CHANGED_SIGNAL],
-				       0);
-			return;
-		}
-	}
+	if (priv->tracks)
+		brasero_burn_session_free_tracks (self);
 
 	sources = priv->pile_tracks->data;
 	priv->pile_tracks = g_slist_remove (priv->pile_tracks, sources);
@@ -1271,17 +1721,15 @@ brasero_burn_session_pop_tracks (BraseroBurnSession *self)
 
 		track = sources->data;
 		brasero_burn_session_start_track_monitoring (self, track);
+		g_signal_emit (self,
+			       brasero_burn_session_signals [TRACK_ADDED_SIGNAL],
+			       0,
+			       track);
 	}
 
-	g_signal_emit (self,
-		       brasero_burn_session_signals [INPUT_CHANGED_SIGNAL],
-		       0);
+	return BRASERO_BURN_RETRY;
 }
 
-/**
- *
- */
-
 gboolean
 brasero_burn_session_is_dest_file (BraseroBurnSession *self)
 {
@@ -1521,8 +1969,8 @@ brasero_burn_session_start (BraseroBurnSession *self)
 
 	type = brasero_track_type_new ();
 	brasero_burn_session_get_input_type (self, type);
-	BRASERO_BURN_LOG_TYPE (type, "Input\t=");
 
+	BRASERO_BURN_LOG_TYPE (type, "Input\t=");
 	BRASERO_BURN_LOG_FLAGS (priv->settings->flags, "flags\t=");
 
 	if (!brasero_burn_session_is_dest_file (self)) {
@@ -1725,7 +2173,19 @@ brasero_burn_session_class_init (BraseroBurnSessionClass *klass)
 	parent_class = g_type_class_peek_parent(klass);
 	object_class->finalize = brasero_burn_session_finalize;
 
+	klass->get_output_path = brasero_burn_session_get_output_path_real;
+	klass->get_output_format = brasero_burn_session_get_output_format_real;
+	klass->set_output_image = brasero_burn_session_set_output_image_real;
+
 	/* This is to delay the setting of track source until we know all settings */
+	/**
+ 	* BraseroBurnSession::output-changed:
+ 	* @session: the object which received the signal
+  	* @former_medium: the medium which was previously set
+	*
+ 	* This signal gets emitted when the medium to burn to changed.
+ 	*
+ 	*/
 	brasero_burn_session_signals [OUTPUT_CHANGED_SIGNAL] =
 	    g_signal_new ("output_changed",
 			  BRASERO_TYPE_BURN_SESSION,
@@ -1737,12 +2197,77 @@ brasero_burn_session_class_init (BraseroBurnSessionClass *klass)
 			  G_TYPE_NONE,
 			  1,
 			  BRASERO_TYPE_MEDIUM);
-
-	brasero_burn_session_signals [INPUT_CHANGED_SIGNAL] =
-	    g_signal_new ("input_changed",
+	/**
+ 	* BraseroBurnSession::track-added:
+ 	* @session: the object which received the signal
+  	* @track: the track that was added
+	*
+ 	* This signal gets emitted when a track is added to @session.
+ 	*
+ 	*/
+	brasero_burn_session_signals [TRACK_ADDED_SIGNAL] =
+	    g_signal_new ("track_added",
 			  BRASERO_TYPE_BURN_SESSION,
 			  G_SIGNAL_RUN_FIRST,
-			  G_STRUCT_OFFSET (BraseroBurnSessionClass, input_changed),
+			  G_STRUCT_OFFSET (BraseroBurnSessionClass, track_added),
+			  NULL,
+			  NULL,
+			  g_cclosure_marshal_VOID__OBJECT,
+			  G_TYPE_NONE,
+			  1,
+			  BRASERO_TYPE_TRACK);
+	/**
+ 	* BraseroBurnSession::track-removed:
+ 	* @session: the object which received the signal
+  	* @track: the track that was removed
+	* @former_position: the former position of @track
+	*
+ 	* This signal gets emitted when a track is removed from @session.
+ 	*
+ 	*/
+	brasero_burn_session_signals [TRACK_REMOVED_SIGNAL] =
+	    g_signal_new ("track_removed",
+			  BRASERO_TYPE_BURN_SESSION,
+			  G_SIGNAL_RUN_FIRST,
+			  G_STRUCT_OFFSET (BraseroBurnSessionClass, track_removed),
+			  NULL,
+			  NULL,
+			  brasero_marshal_VOID__OBJECT_UINT,
+			  G_TYPE_NONE,
+			  2,
+			  BRASERO_TYPE_TRACK,
+			  G_TYPE_UINT);
+	/**
+ 	* BraseroBurnSession::track-changed:
+ 	* @session: the object which received the signal
+  	* @track: the track that changed
+	*
+ 	* This signal gets emitted when the contents of a track changed.
+ 	*
+ 	*/
+	brasero_burn_session_signals [TRACK_CHANGED_SIGNAL] =
+	    g_signal_new ("track_changed",
+			  BRASERO_TYPE_BURN_SESSION,
+			  G_SIGNAL_RUN_FIRST,
+			  G_STRUCT_OFFSET (BraseroBurnSessionClass, track_changed),
+			  NULL,
+			  NULL,
+			  g_cclosure_marshal_VOID__OBJECT,
+			  G_TYPE_NONE,
+			  1,
+			  BRASERO_TYPE_TRACK);
+	/**
+ 	* BraseroBurnSession::flags-changed:
+ 	* @session: the object which received the signal
+	*
+ 	* This signal gets emitted when the flags changed for @session.
+ 	*
+ 	*/
+	brasero_burn_session_signals [FLAGS_CHANGED_SIGNAL] =
+	    g_signal_new ("flags_changed",
+			  BRASERO_TYPE_BURN_SESSION,
+			  G_SIGNAL_RUN_FIRST,
+			  G_STRUCT_OFFSET (BraseroBurnSessionClass, flags_changed),
 			  NULL,
 			  NULL,
 			  g_cclosure_marshal_VOID__VOID,
@@ -1750,6 +2275,14 @@ brasero_burn_session_class_init (BraseroBurnSessionClass *klass)
 			  0);
 }
 
+/**
+ * brasero_burn_session_new:
+ *
+ * Returns a new #BraseroBurnSession object.
+ *
+ * Return value: a #BraseroBurnSession.
+ **/
+
 BraseroBurnSession *
 brasero_burn_session_new ()
 {
