@@ -612,8 +612,7 @@ brasero_job_set_output_file (BraseroJob *self,
 
 			result = brasero_burn_session_get_output (session,
 								  &image,
-								  &toc,
-								  error);
+								  &toc);
 
 			/* check paths are set */
 			if (!image
@@ -1484,7 +1483,7 @@ brasero_job_get_bus_target_lun (BraseroJob *self, gchar **BTL)
 	session = brasero_task_ctx_get_session (priv->ctx);
 
 	drive = brasero_burn_session_get_burner (session);
-	*BTL = brasero_drive_get_bus_target_lun_string (drive);;
+	*BTL = brasero_drive_get_bus_target_lun_string (drive);
 
 	return BRASERO_BURN_OK;
 }
@@ -1891,8 +1890,8 @@ brasero_job_set_rate (BraseroJob *self,
 
 BraseroBurnResult
 brasero_job_set_output_size_for_current_track (BraseroJob *self,
-					       gint64 sectors,
-					       gint64 size)
+					       goffset sectors,
+					       goffset bytes)
 {
 	BraseroJobPrivate *priv;
 
@@ -1912,12 +1911,12 @@ brasero_job_set_output_size_for_current_track (BraseroJob *self,
 
 	return brasero_task_ctx_set_output_size_for_current_track (priv->ctx,
 								   sectors,
-								   size);
+								   bytes);
 }
 
 BraseroBurnResult
 brasero_job_set_written_track (BraseroJob *self,
-			       gint64 written)
+			       goffset written)
 {
 	BraseroJobPrivate *priv;
 
@@ -1933,7 +1932,7 @@ brasero_job_set_written_track (BraseroJob *self,
 
 BraseroBurnResult
 brasero_job_set_written_session (BraseroJob *self,
-				 gint64 written)
+				 goffset written)
 {
 	BraseroJobPrivate *priv;
 
