@@ -35,6 +35,13 @@
 #include <unistd.h>
 #include <string.h>
 
+#ifdef HAVE_CAM_LIB_H
+#include <errno.h>
+#include <stdio.h>
+#include <fcntl.h>
+#include <camlib.h>
+#endif
+
 #include <glib.h>
 #include <glib-object.h>
 #include <glib/gi18n-lib.h>
@@ -70,8 +77,10 @@ struct _BraseroDrivePrivate
 
 	gchar *udi;
 
-	gchar *path;
-	gchar *block_path;
+	gchar *name;
+
+	gchar *device;
+	gchar *block_device;
 
 	GCancellable *cancel;
 
@@ -90,6 +99,7 @@ static gulong drive_signals [LAST_SIGNAL] = {0, };
 
 enum {
 	PROP_NONE	= 0,
+	PROP_DEVICE,
 	PROP_GDRIVE,
 	PROP_UDI
 };
@@ -119,6 +129,9 @@ brasero_drive_get_gdrive (BraseroDrive *drive)
 
 	priv = BRASERO_DRIVE_PRIVATE (drive);
 
+	if (!priv->gdrive)
+		return NULL;
+
 	return g_object_ref (priv->gdrive);
 }
 
@@ -196,6 +209,13 @@ brasero_drive_eject (BraseroDrive *drive,
 
 	priv = BRASERO_DRIVE_PRIVATE (drive);
 
+	/* reset if needed */
+	if (g_cancellable_is_cancelled (priv->cancel)) {
+		BRASERO_MEDIA_LOG ("Resetting GCancellable object");
+		g_cancellable_reset (priv->cancel);
+	}
+
+	BRASERO_MEDIA_LOG ("Trying to eject drive (?)");
 	res = brasero_gio_operation_eject_drive (priv->gdrive,
 						 priv->cancel,
 						 wait,
@@ -206,6 +226,13 @@ brasero_drive_eject (BraseroDrive *drive,
 	if (!priv->medium)
 		return FALSE;
 
+	/* reset if needed */
+	if (g_cancellable_is_cancelled (priv->cancel)) {
+		BRASERO_MEDIA_LOG ("Resetting GCancellable object");
+		g_cancellable_reset (priv->cancel);
+	}
+
+	BRASERO_MEDIA_LOG ("Trying to eject volume");
 	gvolume = brasero_volume_get_gvolume (BRASERO_VOLUME (priv->medium));
 	res = brasero_gio_operation_eject_volume (gvolume,
 						  priv->cancel,
@@ -231,7 +258,9 @@ brasero_drive_cancel_current_operation (BraseroDrive *drive)
 	g_return_if_fail (drive != NULL);
 	g_return_if_fail (BRASERO_IS_DRIVE (drive));
 
-	priv = BRASERO_DRIVE_PRIVATE (drive);	
+	priv = BRASERO_DRIVE_PRIVATE (drive);
+
+	BRASERO_MEDIA_LOG ("Cancelling GIO operation");
 	g_cancellable_cancel (priv->cancel);
 }
 
@@ -242,9 +271,13 @@ brasero_drive_cancel_current_operation (BraseroDrive *drive)
  * Returns the bus, target, lun ("{bus},{target},{lun}") as a string which is
  * sometimes needed by some backends like cdrecord.
  *
+ * NOTE: that function returns either bus/target/lun or the device path
+ * according to OSes. Basically it returns bus/target/lun only for FreeBSD
+ * which is the only OS in need for that. For all others it returns the device
+ * path. 
+ *
  * Return value: a string or NULL. The string must be freed when not needed
  *
- * Deprecated since 2.27.3
  **/
 gchar *
 brasero_drive_get_bus_target_lun_string (BraseroDrive *drive)
@@ -252,7 +285,7 @@ brasero_drive_get_bus_target_lun_string (BraseroDrive *drive)
 	g_return_val_if_fail (drive != NULL, NULL);
 	g_return_val_if_fail (BRASERO_IS_DRIVE (drive), NULL);
 
-	return NULL;
+	return brasero_device_get_bus_target_lun (brasero_drive_get_device (drive));
 }
 
 /**
@@ -273,7 +306,7 @@ brasero_drive_is_fake (BraseroDrive *drive)
 	g_return_val_if_fail (BRASERO_IS_DRIVE (drive), FALSE);
 
 	priv = BRASERO_DRIVE_PRIVATE (drive);
-	return (priv->gdrive == NULL);
+	return (priv->device == NULL);
 }
 
 /**
@@ -287,6 +320,7 @@ brasero_drive_is_fake (BraseroDrive *drive)
 gboolean
 brasero_drive_is_door_open (BraseroDrive *drive)
 {
+	const gchar *device;
 	BraseroDrivePrivate *priv;
 	BraseroDeviceHandle *handle;
 	BraseroScsiMechStatusHdr hdr;
@@ -295,10 +329,11 @@ brasero_drive_is_door_open (BraseroDrive *drive)
 	g_return_val_if_fail (BRASERO_IS_DRIVE (drive), FALSE);
 
 	priv = BRASERO_DRIVE_PRIVATE (drive);
-	if (!priv->gdrive)
+	if (!priv->device)
 		return FALSE;
 
-	handle = brasero_device_handle_open (priv->path, FALSE, NULL);
+	device = brasero_drive_get_block_device (drive);
+	handle = brasero_device_handle_open (device, FALSE, NULL);
 	if (!handle)
 		return FALSE;
 
@@ -329,8 +364,7 @@ brasero_drive_can_use_exclusively (BraseroDrive *drive)
 	g_return_val_if_fail (drive != NULL, FALSE);
 	g_return_val_if_fail (BRASERO_IS_DRIVE (drive), FALSE);
 
-	device = brasero_drive_get_device (drive);
-
+	device = brasero_drive_get_block_device (drive);
 	handle = brasero_device_handle_open (device, TRUE, NULL);
 	if (!handle)
 		return FALSE;
@@ -363,10 +397,10 @@ brasero_drive_lock (BraseroDrive *drive,
 	g_return_val_if_fail (BRASERO_IS_DRIVE (drive), FALSE);
 
 	priv = BRASERO_DRIVE_PRIVATE (drive);
-	if (!priv->gdrive)
+	if (!priv->device)
 		return FALSE;
 
-	device = brasero_drive_get_device (drive);
+	device = brasero_drive_get_block_device (drive);
 	handle = brasero_device_handle_open (device, FALSE, NULL);
 	if (!handle)
 		return FALSE;
@@ -402,10 +436,10 @@ brasero_drive_unlock (BraseroDrive *drive)
 	g_return_val_if_fail (BRASERO_IS_DRIVE (drive), FALSE);
 
 	priv = BRASERO_DRIVE_PRIVATE (drive);
-	if (!priv->gdrive)
+	if (!!priv->device)
 		return FALSE;
 
-	device = brasero_drive_get_device (drive);
+	device = brasero_drive_get_block_device (drive);
 	handle = brasero_device_handle_open (device, FALSE, NULL);
 	if (!handle)
 		return FALSE;
@@ -439,14 +473,17 @@ brasero_drive_get_display_name (BraseroDrive *drive)
 	g_return_val_if_fail (BRASERO_IS_DRIVE (drive), NULL);
 
 	priv = BRASERO_DRIVE_PRIVATE (drive);
-	if (!priv->gdrive) {
+	if (!priv->device) {
 		/* Translators: This is a fake drive, a file, and means that
 		 * when we're writing, we're writing to a file and create an
 		 * image on the hard drive. */
 		return g_strdup (_("Image File"));
 	}
 
-	return g_drive_get_name (priv->gdrive);
+	if (priv->gdrive)
+		return g_drive_get_name (priv->gdrive);
+
+	return g_strdup (priv->name);
 }
 
 /**
@@ -466,7 +503,7 @@ brasero_drive_get_device (BraseroDrive *drive)
 	g_return_val_if_fail (BRASERO_IS_DRIVE (drive), NULL);
 
 	priv = BRASERO_DRIVE_PRIVATE (drive);
-	return priv->path? priv->path:priv->block_path;
+	return priv->device;
 }
 
 /**
@@ -474,9 +511,11 @@ brasero_drive_get_device (BraseroDrive *drive)
  * @drive: a #BraseroDrive
  *
  * Gets a string holding the block device path for the drive. This can be used on
- * some other OS, like Solaris, for burning operations instead of the device
+ * some other OSes, like Solaris, for burning operations instead of the device
  * path.
  *
+ * If such a path is not available, it returns the device path.
+ *
  * Return value: a string holding the block device path
  **/
 const gchar *
@@ -488,7 +527,7 @@ brasero_drive_get_block_device (BraseroDrive *drive)
 	g_return_val_if_fail (BRASERO_IS_DRIVE (drive), NULL);
 
 	priv = BRASERO_DRIVE_PRIVATE (drive);
-	return priv->block_path;
+	return priv->block_device? priv->block_device:priv->device;
 }
 
 /**
@@ -498,8 +537,7 @@ brasero_drive_get_block_device (BraseroDrive *drive)
  * Gets a string holding the HAL udi corresponding to this device. It can be used
  * to uniquely identify the drive.
  *
- * Return value: a string holding the HAL udi. Not to be freed
- * Deprecated since 2.27.3
+ * Return value: a string holding the HAL udi or NULL. Not to be freed
  **/
 const gchar *
 brasero_drive_get_udi (BraseroDrive *drive)
@@ -512,7 +550,7 @@ brasero_drive_get_udi (BraseroDrive *drive)
 	g_return_val_if_fail (BRASERO_IS_DRIVE (drive), NULL);
 
 	priv = BRASERO_DRIVE_PRIVATE (drive);
-	if (!priv->gdrive)
+	if (!priv->device || !priv->gdrive)
 		return NULL;
 
 	if (priv->udi)
@@ -542,8 +580,7 @@ brasero_drive_get_medium (BraseroDrive *drive)
 	g_return_val_if_fail (BRASERO_IS_DRIVE (drive), NULL);
 
 	priv = BRASERO_DRIVE_PRIVATE (drive);
-
-	if (!priv->probed && priv->gdrive)
+	if (!priv->probed)
 		return NULL;
 
 	return priv->medium;
@@ -569,6 +606,73 @@ brasero_drive_get_caps (BraseroDrive *drive)
 	return priv->caps;
 }
 
+/**
+ * brasero_drive_can_write_media:
+ * @drive: a #BraseroDrive
+ * @media: a #BraseroMedia
+ *
+ * Returns whether the disc can burn a specific media type.
+ *
+ * Return value: a #gboolean. TRUE if the drive can write this type of media and FALSE otherwise
+ **/
+gboolean
+brasero_drive_can_write_media (BraseroDrive *drive,
+                               BraseroMedia media)
+{
+	BraseroDrivePrivate *priv;
+
+	g_return_val_if_fail (drive != NULL, FALSE);
+	g_return_val_if_fail (BRASERO_IS_DRIVE (drive), FALSE);
+
+	priv = BRASERO_DRIVE_PRIVATE (drive);
+
+	if (!(media & BRASERO_MEDIUM_REWRITABLE)
+	&&   (media & BRASERO_MEDIUM_CLOSED))
+		return FALSE;
+
+	if (media & BRASERO_MEDIUM_FILE)
+		return FALSE;
+
+	if (BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_CDR))
+		return (priv->caps & BRASERO_DRIVE_CAPS_CDR) != 0;
+
+	if (BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDR))
+		return (priv->caps & BRASERO_DRIVE_CAPS_DVDR) != 0;
+
+	if (BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDR_PLUS))
+		return (priv->caps & BRASERO_DRIVE_CAPS_DVDR_PLUS) != 0;
+
+	if (BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_CDRW))
+		return (priv->caps & BRASERO_DRIVE_CAPS_CDRW) != 0;
+
+	if (BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW))
+		return (priv->caps & BRASERO_DRIVE_CAPS_DVDRW) != 0;
+
+	if (BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW_RESTRICTED))
+		return (priv->caps & BRASERO_DRIVE_CAPS_DVDRW) != 0;
+
+	if (BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW_PLUS))
+		return (priv->caps & BRASERO_DRIVE_CAPS_DVDRW_PLUS) != 0;
+
+	if (BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDR_PLUS_DL))
+		return (priv->caps & BRASERO_DRIVE_CAPS_DVDR_PLUS_DL) != 0;
+
+	if (BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW_PLUS_DL))
+		return (priv->caps & BRASERO_DRIVE_CAPS_DVDRW_PLUS_DL) != 0;
+
+	if (BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVD_RAM))
+		return (priv->caps & BRASERO_DRIVE_CAPS_DVDRAM) != 0;
+
+	/* All types of BD-R */
+	if (BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_BD|BRASERO_MEDIUM_WRITABLE))
+		return (priv->caps & BRASERO_DRIVE_CAPS_BDR) != 0;
+
+	if (BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_BDRE))
+		return (priv->caps & BRASERO_DRIVE_CAPS_BDRW) != 0;
+
+	return FALSE;
+}
+
 /**
  * brasero_drive_can_write:
  * @drive: a #BraseroDrive
@@ -658,6 +762,12 @@ brasero_drive_reprobe (BraseroDrive *drive)
 	g_return_if_fail (BRASERO_IS_DRIVE (drive));
 
 	priv = BRASERO_DRIVE_PRIVATE (drive);
+
+	if (priv->gdrive) {
+		/* reprobe the contents of the drive system wide */
+		g_drive_poll_for_media (priv->gdrive, NULL, NULL, NULL);
+	}
+
 	if (!priv->medium)
 		return;
 
@@ -735,17 +845,12 @@ brasero_drive_medium_gdrive_changed_cb (BraseroDrive *gdrive,
 	brasero_drive_check_medium_inside_gdrive (drive);
 }
 
-static gboolean
-brasero_drive_probed (gpointer data)
+static void
+brasero_drive_update_gdrive (BraseroDrive *drive)
 {
-	BraseroDrive *drive = BRASERO_DRIVE (data);
 	BraseroDrivePrivate *priv;
 
-	priv = BRASERO_DRIVE_PRIVATE (data);
-
-	g_thread_join (priv->probe);
-	priv->probe = NULL;
-
+	priv = BRASERO_DRIVE_PRIVATE (drive);
 	/* If it's not a fake drive then connect to signal for any
 	 * change and check medium inside */
 	g_signal_connect (priv->gdrive,
@@ -754,8 +859,19 @@ brasero_drive_probed (gpointer data)
 			  drive);
 
 	brasero_drive_check_medium_inside_gdrive (drive);
+}
 
+static gboolean
+brasero_drive_probed (gpointer data)
+{
+	BraseroDrivePrivate *priv;
+
+	priv = BRASERO_DRIVE_PRIVATE (data);
+
+	g_thread_join (priv->probe);
+	priv->probe = NULL;
 	priv->probe_id = 0;
+
 	return FALSE;
 }
 
@@ -785,7 +901,7 @@ brasero_drive_get_caps_profiles (BraseroDrive *self,
 		return FALSE;
 	}
 
-	BRASERO_MEDIA_LOG ("Dectected media %x", BRASERO_GET_16 (hdr->current_profile));
+	BRASERO_MEDIA_LOG ("Dectected medium is 0x%x", BRASERO_GET_16 (hdr->current_profile));
 
 	/* Go through all features available */
 	desc = hdr->desc;
@@ -886,22 +1002,20 @@ static gpointer
 brasero_drive_probe_thread (gpointer data)
 {
 	gint counter = 0;
-	const gchar *path;
+	const gchar *device;
 	BraseroScsiErrCode code;
 	BraseroDrivePrivate *priv;
 	BraseroDeviceHandle *handle;
 	BraseroDrive *drive = BRASERO_DRIVE (data);
 
 	priv = BRASERO_DRIVE_PRIVATE (drive);
-	path = brasero_drive_get_device (drive);
-	if (!path)
-		path = brasero_drive_get_block_device (drive);
 
 	/* the drive might be busy (a burning is going on) so we don't block
 	 * but we re-try to open it every second */
-	BRASERO_MEDIA_LOG ("Trying to open device %s", path);
+	device = brasero_drive_get_block_device (drive);
+	BRASERO_MEDIA_LOG ("Trying to open device %s", device);
 
-	handle = brasero_device_handle_open (path, FALSE, &code);
+	handle = brasero_device_handle_open (device, FALSE, &code);
 	while (!handle && counter <= BRASERO_DRIVE_OPEN_ATTEMPTS) {
 		sleep (1);
 
@@ -912,7 +1026,7 @@ brasero_drive_probe_thread (gpointer data)
 		}
 
 		counter ++;
-		handle = brasero_device_handle_open (path, FALSE, &code);
+		handle = brasero_device_handle_open (device, FALSE, &code);
 	}
 
 	if (priv->probe_cancelled) {
@@ -922,8 +1036,40 @@ brasero_drive_probe_thread (gpointer data)
 	}
 
 	if (handle) {
+		BraseroScsiInquiry hdr;
+		BraseroScsiResult res;
+
 		BRASERO_MEDIA_LOG ("Open () succeeded");
 
+		/* get additional information like the name */
+		res = brasero_spc1_inquiry (handle, &hdr, NULL);
+		if (res == BRASERO_SCSI_OK) {
+			gchar *name_utf8;
+			gchar *vendor;
+			gchar *model;
+			gchar *name;
+
+			vendor = strndup ((gchar *) hdr.vendor, sizeof (hdr.vendor));
+			model = strndup ((gchar *) hdr.name, sizeof (hdr.name));
+			name = g_strdup_printf ("%s %s", g_strstrip (vendor), g_strstrip (model));
+			g_free (vendor);
+			g_free (model);
+
+			/* make sure that's proper UTF-8 */
+			name_utf8 = g_convert_with_fallback (name,
+			                                     -1,
+			                                     "ASCII",
+			                                     "UTF-8",
+			                                     "_",
+			                                     NULL,
+			                                     NULL,
+			                                     NULL);
+			g_free (name);
+
+			priv->name = name_utf8;
+		}
+
+		/* Get supported medium types */
 		if (!brasero_drive_get_caps_profiles (drive, handle, &code))
 			brasero_drive_get_caps_2A (drive, handle, &code);
 
@@ -939,15 +1085,16 @@ brasero_drive_probe_thread (gpointer data)
 }
 
 static void
-brasero_drive_init_real (BraseroDrive *drive)
+brasero_drive_init_real_device (BraseroDrive *drive,
+                                const gchar *device)
 {
 	BraseroDrivePrivate *priv;
 
 	priv = BRASERO_DRIVE_PRIVATE (drive);
 
-	priv->block_path = g_drive_get_identifier (priv->gdrive, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
+	priv->device = g_strdup (device);
 
-	BRASERO_MEDIA_LOG ("Initializing drive %s", priv->block_path);
+	BRASERO_MEDIA_LOG ("Initializing drive %s from device", priv->device);
 
 	/* NOTE: why a thread? Because in case of a damaged medium, brasero can
 	 * block on some functions until timeout and if we do this in the main
@@ -967,6 +1114,7 @@ brasero_drive_set_property (GObject *object,
 			    GParamSpec *pspec)
 {
 	BraseroDrivePrivate *priv;
+	GDrive *gdrive = NULL;
 
 	g_return_if_fail (BRASERO_IS_DRIVE (object));
 
@@ -977,16 +1125,44 @@ brasero_drive_set_property (GObject *object,
 	case PROP_UDI:
 		break;
 	case PROP_GDRIVE:
-		priv->gdrive = g_value_get_object (value);
-		if (!priv->gdrive) {
-			priv->probed = TRUE;
+		if (!priv->device)
+			break;
+
+		gdrive = g_value_get_object (value);
+		if (priv->gdrive) {
+			g_signal_handlers_disconnect_by_func (priv->gdrive,
+							      brasero_drive_medium_gdrive_changed_cb,
+							      object);
+			g_object_unref (priv->gdrive);
+		}
+
+		BRASERO_MEDIA_LOG ("Setting GDrive %p", gdrive);
+
+		if (gdrive) {
+			priv->gdrive = g_object_ref (gdrive);
+			brasero_drive_update_gdrive (BRASERO_DRIVE (object));
+		}
+		else if (!priv->medium) {
+			priv->probed = FALSE;
 			priv->medium = g_object_new (BRASERO_TYPE_VOLUME,
 						     "drive", object,
 						     NULL);
+
+			g_signal_connect (priv->medium,
+					  "probed",
+					  G_CALLBACK (brasero_drive_medium_probed),
+					  object);
+		}
+		break;
+	case PROP_DEVICE:
+		if (!g_value_get_string (value)) {
+			priv->medium = g_object_new (BRASERO_TYPE_VOLUME,
+						     "drive", object,
+						     NULL);
+			priv->probed = TRUE;
 		}
 		else
-			brasero_drive_init_real (BRASERO_DRIVE (object));
-
+			brasero_drive_init_real_device (BRASERO_DRIVE (object), g_value_get_string (value));
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -1013,6 +1189,9 @@ brasero_drive_get_property (GObject *object,
 	case PROP_GDRIVE:
 		g_value_set_object (value, priv->gdrive);
 		break;
+	case PROP_DEVICE:
+		g_value_set_string (value, priv->device);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -1026,10 +1205,7 @@ brasero_drive_finalize (GObject *object)
 
 	priv = BRASERO_DRIVE_PRIVATE (object);
 
-	if (priv->udi) {
-		g_free (priv->udi);
-		priv->udi = NULL;
-	}
+	BRASERO_MEDIA_LOG ("Finalizing BraseroDrive");
 
 	if (priv->probe) {
 		priv->probe_cancelled = TRUE;
@@ -1042,16 +1218,35 @@ brasero_drive_finalize (GObject *object)
 		priv->probe_id = 0;
 	}
 
-	if (priv->block_path) {
-		g_free (priv->block_path);
-		priv->block_path = NULL;
-	}
-
 	if (priv->medium) {
+		g_signal_emit (object,
+			       drive_signals [MEDIUM_REMOVED],
+			       0,
+			       priv->medium);
 		g_object_unref (priv->medium);
 		priv->medium = NULL;
 	}
 
+	if (priv->name) {
+		g_free (priv->name);
+		priv->name = NULL;
+	}
+
+	if (priv->device) {
+		g_free (priv->device);
+		priv->device = NULL;
+	}
+
+	if (priv->block_device) {
+		g_free (priv->block_device);
+		priv->block_device = NULL;
+	}
+
+	if (priv->udi) {
+		g_free (priv->udi);
+		priv->udi = NULL;
+	}
+
 	if (priv->gdrive) {
 		g_signal_handlers_disconnect_by_func (priv->gdrive,
 						      brasero_drive_medium_gdrive_changed_cb,
@@ -1119,7 +1314,7 @@ brasero_drive_class_init (BraseroDriveClass *klass)
 	g_object_class_install_property (object_class,
 	                                 PROP_UDI,
 	                                 g_param_spec_string("udi",
-	                                                     "Deprecated",
+	                                                     "udi",
 	                                                     "HAL udi as a string (Deprecated)",
 	                                                     NULL,
 	                                                     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
@@ -1129,6 +1324,13 @@ brasero_drive_class_init (BraseroDriveClass *klass)
 	                                                      "GDrive",
 	                                                      "A GDrive object for the drive",
 	                                                      G_TYPE_DRIVE,
-	                                                     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	                                                     G_PARAM_READWRITE));
+	g_object_class_install_property (object_class,
+	                                 PROP_DEVICE,
+	                                 g_param_spec_string ("device",
+	                                                      "Device",
+	                                                      "Device path for the drive",
+	                                                      NULL,
+	                                                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 }
 
