@@ -74,6 +74,9 @@ enum {
 	MEDIUM_COL,
 	NAME_COL,
 	ICON_COL,
+	USED_COL,
+	VISIBLE_PROGRESS_COL,
+	VISIBLE_TEXT_COL,
 	NUM_COL
 };
 
@@ -104,9 +107,12 @@ brasero_medium_selection_foreach (BraseroMediumSelection *selection,
 		if (!medium)
 			return;
 
-		if (!function (medium, callback_data))
+		if (!function (medium, callback_data)) {
+			g_object_unref (medium);
 			break;
+		}
 
+		g_object_unref (medium);
 	} while (gtk_tree_model_iter_next (model, &iter));
 }
 
@@ -176,6 +182,79 @@ brasero_medium_selection_get_medium_string (BraseroMediumSelection *self,
 	return label;
 }
 
+void
+brasero_medium_selection_update_used_space (BraseroMediumSelection *selector,
+					    BraseroMedium *medium_arg,
+					    guint used_space)
+{
+	BraseroMediumSelectionPrivate *priv;
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+
+	priv = BRASERO_MEDIUM_SELECTION_PRIVATE (selector);
+
+	model = gtk_combo_box_get_model (GTK_COMBO_BOX (selector));
+	if (!gtk_tree_model_get_iter_first (model, &iter))
+		return;
+
+	do {
+		BraseroMedium *medium;
+
+		medium = NULL;
+		gtk_tree_model_get (model, &iter,
+				    MEDIUM_COL, &medium,
+				    -1);
+
+		if (medium == medium_arg) {
+			g_object_unref (medium);
+
+			gtk_list_store_set (GTK_LIST_STORE (model), &iter,
+					    USED_COL, used_space,
+					    VISIBLE_PROGRESS_COL, (gboolean) (used_space > 0),
+					    VISIBLE_TEXT_COL, (gboolean) (used_space <= 0),
+					    -1);
+			break;
+		}
+
+		g_object_unref (medium);
+	} while (gtk_tree_model_iter_next (model, &iter));
+}
+
+static void
+brasero_medium_selection_set_show_used_space (BraseroMediumSelection *selector)
+{
+	GtkCellRenderer *renderer;
+	BraseroMediumSelectionPrivate *priv;
+
+	priv = BRASERO_MEDIUM_SELECTION_PRIVATE (selector);
+
+	gtk_cell_layout_clear (GTK_CELL_LAYOUT (selector));
+
+	renderer = gtk_cell_renderer_pixbuf_new ();
+	g_object_set (renderer, "follow-state", TRUE, NULL);
+	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (selector), renderer, FALSE);
+	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (selector), renderer,
+					"gicon", ICON_COL,
+					NULL);
+
+	renderer = gtk_cell_renderer_progress_new ();
+	g_object_set (renderer, "xpad", 4, "text-xalign", 0.0, NULL);
+	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (selector), renderer, TRUE);
+	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (selector), renderer,
+					"text", NAME_COL,
+					"value", USED_COL,
+					"visible", VISIBLE_PROGRESS_COL,
+					NULL);
+
+	renderer = gtk_cell_renderer_text_new ();
+	g_object_set (renderer, "xpad", 4, NULL);
+	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (selector), renderer, TRUE);
+	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (selector), renderer,
+					"markup", NAME_COL,
+					"visible", VISIBLE_TEXT_COL,
+					NULL);
+}
+
 void
 brasero_medium_selection_update_media_string (BraseroMediumSelection *self)
 {
@@ -228,17 +307,19 @@ brasero_medium_selection_set_current_medium (BraseroMediumSelection *self,
 	else
 		gtk_widget_set_sensitive (GTK_WIDGET (self), FALSE);
 
-	if (priv->active == medium)
+	if (priv->active == medium) {
+		if (medium)
+			g_object_unref (medium);
+
 		return;
+	}
 
 	if (priv->active)
 		g_object_unref (priv->active);
 
+	/* NOTE: no need to ref priv->active as medium was 
+	 * already reffed in gtk_tree_model_get () */
 	priv->active = medium;
-
-	if (priv->active)
-		g_object_ref (priv->active);
-
 	g_signal_emit (self,
 		       brasero_medium_selection_signals [CHANGED_SIGNAL],
 		       0,
@@ -370,6 +451,8 @@ brasero_medium_selection_update_no_disc_entry (BraseroMediumSelection *self,
 		gtk_list_store_set (GTK_LIST_STORE (model), iter,
 				    NAME_COL, _("Searching for available discs"),
 				    ICON_COL, icon,
+				    VISIBLE_TEXT_COL, TRUE,
+				    VISIBLE_PROGRESS_COL, FALSE,
 				    -1);
 	}
 	else {
@@ -377,6 +460,8 @@ brasero_medium_selection_update_no_disc_entry (BraseroMediumSelection *self,
 		gtk_list_store_set (GTK_LIST_STORE (model), iter,
 				    NAME_COL, _("No available disc"),
 				    ICON_COL, icon,
+				    VISIBLE_TEXT_COL, TRUE,
+				    VISIBLE_PROGRESS_COL, FALSE,
 				    -1);
 	}
 
@@ -477,16 +562,21 @@ brasero_medium_selection_show_media_type (BraseroMediumSelection *selector,
 
 			medium = item->data;
 
-			medium_name = brasero_medium_selection_get_medium_string (selector, medium);
-			medium_icon = brasero_volume_get_icon (BRASERO_VOLUME (medium));
-
 			gtk_list_store_append (GTK_LIST_STORE (model), &iter);
 			gtk_list_store_set (GTK_LIST_STORE (model), &iter,
 					    MEDIUM_COL, medium,
+					    -1);
+
+			medium_name = brasero_medium_selection_get_medium_string (selector, medium);
+			medium_icon = brasero_volume_get_icon (BRASERO_VOLUME (medium));
+			gtk_list_store_set (GTK_LIST_STORE (model), &iter,
 					    NAME_COL, medium_name,
 					    ICON_COL, medium_icon,
+					    VISIBLE_TEXT_COL, TRUE,
+					    VISIBLE_PROGRESS_COL, FALSE,
 					    -1);
 			g_free (medium_name);
+			g_object_unref (medium_icon);
 		}
 		g_slist_foreach (list, (GFunc) g_object_unref, NULL);
 		g_slist_free (list);
@@ -583,6 +673,8 @@ brasero_medium_selection_medium_added_cb (BraseroMediumMonitor *monitor,
 	model = gtk_combo_box_get_model (GTK_COMBO_BOX (self));
 
 	if (!add) {
+		BraseroMedium *tmp;
+
 		/* Try to get the first iter (it shouldn't fail) */
 		if (!gtk_tree_model_get_iter_first (model, &iter)) {
 			brasero_medium_selection_add_no_disc_entry (self);
@@ -590,12 +682,14 @@ brasero_medium_selection_medium_added_cb (BraseroMediumMonitor *monitor,
 		}
 
 		/* See if that's a real medium or not; if so, return. */
-		medium = NULL;
+		tmp = NULL;
 		gtk_tree_model_get (model, &iter,
-				    MEDIUM_COL, &medium,
+				    MEDIUM_COL, &tmp,
 				    -1);
-		if (medium)
+		if (tmp) {
+			g_object_unref (tmp);
 			return;
+		}
 
 		brasero_medium_selection_update_no_disc_entry (self, model, &iter);
 		return;
@@ -614,15 +708,21 @@ brasero_medium_selection_medium_added_cb (BraseroMediumMonitor *monitor,
 			g_object_unref (tmp);
 	}
 
-	medium_name = brasero_medium_selection_get_medium_string (self, medium);
-	medium_icon = brasero_volume_get_icon (BRASERO_VOLUME (medium));
 	gtk_list_store_append (GTK_LIST_STORE (model), &iter);
 	gtk_list_store_set (GTK_LIST_STORE (model), &iter,
 			    MEDIUM_COL, medium,
+			    -1);
+
+	medium_name = brasero_medium_selection_get_medium_string (self, medium);
+	medium_icon = brasero_volume_get_icon (BRASERO_VOLUME (medium));
+	gtk_list_store_set (GTK_LIST_STORE (model), &iter,
 			    NAME_COL, medium_name,
 			    ICON_COL, medium_icon,
+			    VISIBLE_TEXT_COL, TRUE,
+			    VISIBLE_PROGRESS_COL, FALSE,
 			    -1);
 	g_free (medium_name);
+	g_object_unref (medium_icon);
 
 	gtk_widget_set_sensitive (GTK_WIDGET (self), TRUE);
 	if (gtk_combo_box_get_active (GTK_COMBO_BOX (self)) == -1) {
@@ -677,7 +777,6 @@ static void
 brasero_medium_selection_init (BraseroMediumSelection *object)
 {
 	GtkListStore *model;
-	GtkCellRenderer *renderer;
 	BraseroMediumMonitor *monitor;
 	BraseroMediumSelectionPrivate *priv;
 
@@ -699,24 +798,15 @@ brasero_medium_selection_init (BraseroMediumSelection *object)
 	model = gtk_list_store_new (NUM_COL,
 				    G_TYPE_OBJECT,
 				    G_TYPE_STRING,
-				    G_TYPE_ICON);
+				    G_TYPE_ICON,
+				    G_TYPE_UINT,
+				    G_TYPE_BOOLEAN,
+				    G_TYPE_BOOLEAN);
 
 	gtk_combo_box_set_model (GTK_COMBO_BOX (object), GTK_TREE_MODEL (model));
 	g_object_unref (model);
 
-	renderer = gtk_cell_renderer_pixbuf_new ();
-	g_object_set (renderer, "follow-state", TRUE, NULL);
-	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (object), renderer, FALSE);
-	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (object), renderer,
-					"gicon", ICON_COL,
-					NULL);
-
-	renderer = gtk_cell_renderer_text_new ();
-	g_object_set (renderer, "xpad", 8, NULL);
-	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (object), renderer, TRUE);
-	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (object), renderer,
-					"markup", NAME_COL,
-					NULL);
+	brasero_medium_selection_set_show_used_space (object);
 }
 
 static void
@@ -840,6 +930,13 @@ brasero_medium_selection_class_init (BraseroMediumSelectionClass *klass)
 			  BRASERO_TYPE_MEDIUM);
 }
 
+/**
+ * brasero_medium_selection_new:
+ *
+ * Creates a new #BraseroMediumSelection object
+ *
+ * Return value: a #GtkWidget. Unref when it is not needed anymore.
+ **/
 GtkWidget *
 brasero_medium_selection_new (void)
 {
