@@ -115,6 +115,9 @@ struct _BraseroMediumPrivate
 	guint dummy_tao:1;
 	guint burnfree:1;
 
+	guint blank_command:1;
+	guint write_command:1;
+
 	guint probe_cancelled:1;
 };
 
@@ -290,13 +293,15 @@ brasero_medium_get_status (BraseroMedium *medium)
 /**
  * brasero_medium_get_last_data_track_address:
  * @medium: #BraseroMedium
- * @byte: a #goffset * or NULL
- * @sector: a #goffset * or NULL
+ * @bytes: a #goffset * or NULL
+ * @sectors: a #goffset * or NULL
  *
- * Stores in either @byte (in bytes) or in @sector (in blocks) the address where
+ * Stores in either @bytes (in bytes) or in @sectors (in blocks) the address where
  * the last session starts. This is useful when creating a multisession image or
  * when reading the contents of this last track.
  *
+ * Return value: a #gboolean. Returns TRUE if information could be retrieved.
+ *
  **/
 gboolean
 brasero_medium_get_last_data_track_address (BraseroMedium *medium,
@@ -335,12 +340,14 @@ brasero_medium_get_last_data_track_address (BraseroMedium *medium,
 /**
  * brasero_medium_get_last_data_track_space:
  * @medium: #BraseroMedium
- * @size: a #goffset * or NULL
- * @blocks: a #goffset * or NULL
+ * @bytes: a #goffset * or NULL
+ * @sectors: a #goffset * or NULL
  *
- * Stores in either @size (in bytes) or in @blocks (in blocks) the space used by
+ * Stores in either @bytes (in bytes) or in @sectors (in blocks) the space used by
  * the last track on the medium.
  *
+ * Return value: a #gboolean. Returns TRUE if information could be retrieved.
+ *
  **/
 gboolean
 brasero_medium_get_last_data_track_space (BraseroMedium *medium,
@@ -443,10 +450,10 @@ brasero_medium_get_track (BraseroMedium *medium,
  * brasero_medium_get_track_space:
  * @medium: a #BraseroMedium
  * @num: a #guint
- * @size: a #goffset * or NULL
- * @blocks: a #goffset * or NULL
+ * @bytes: a #goffset * or NULL
+ * @sectors: a #goffset * or NULL
  *
- * Stores in either @size (in bytes) or in @blocks (in blocks) the space used
+ * Stores in either @bytes (in bytes) or in @sectors (in blocks) the space used
  * by session @num on the disc.
  *
  * Return value: a #gboolean. Returns TRUE if information could be retrieved;
@@ -488,10 +495,10 @@ brasero_medium_get_track_space (BraseroMedium *medium,
  * brasero_medium_get_track_address:
  * @medium: a #BraseroMedium
  * @num: a #guint
- * @byte: a #goffset * or NULL
- * @sector: a #goffset * or NULL
+ * @bytes: a #goffset * or NULL
+ * @sectors: a #goffset * or NULL
  *
- * Stores in either @byte (in bytes) or in @sector (in blocks) the address at
+ * Stores in either @bytes (in bytes) or in @sectors (in blocks) the address at
  * which the session identified by @num starts.
  *
  * Return value: a #gboolean. Returns TRUE if information could be retrieved;
@@ -858,6 +865,12 @@ brasero_medium_test_simulate_CD_TAO (BraseroMedium *self,
 	tao_desc = (BraseroScsiCDTAODesc *) desc->data;
 	priv->dummy_tao = tao_desc->dummy != 0;
 	priv->burnfree = tao_desc->buf != 0;
+
+	/* See if CD-RW is supported which means in
+	 * this case that we can blank */
+	priv->blank_command = (tao_desc->CDRW != 0);
+	BRASERO_MEDIA_LOG ("Medium %s be blanked", priv->blank_command? "can":"cannot");
+
 	g_free (hdr);
 	return TRUE;
 }
@@ -894,6 +907,7 @@ brasero_medium_test_simulate_CD_SAO (BraseroMedium *self,
 	sao_desc = (BraseroScsiCDSAODesc *) desc->data;
 	priv->dummy_sao = sao_desc->dummy != 0;
 	priv->burnfree = sao_desc->buf != 0;
+
 	g_free (hdr);
 	return TRUE;
 }
@@ -932,6 +946,12 @@ brasero_medium_test_simulate_DVDRW (BraseroMedium *self,
 	priv->dummy_sao = less_wrt_desc->dummy != 0;
 	priv->dummy_tao = less_wrt_desc->dummy != 0;
 	priv->burnfree = less_wrt_desc->buf != 0;
+
+	/* NOTE: it's said that this is only valid when the current
+	 * bit is set which is always the case in this function */
+	priv->blank_command = (less_wrt_desc->rw_DVD != 0);
+	BRASERO_MEDIA_LOG ("Medium %s be blanked", priv->blank_command? "can":"cannot");
+
 	g_free (hdr);
 	return TRUE;
 }
@@ -971,6 +991,10 @@ brasero_medium_test_simulate_2A (BraseroMedium *self,
 	priv->dummy_sao = page_2A->dummy != 0;
 	priv->dummy_tao = page_2A->dummy != 0;
 	priv->burnfree = page_2A->buffer != 0;
+
+	priv->blank_command = (page_2A->wr_CDRW != 0);
+	BRASERO_MEDIA_LOG ("Medium %s be blanked", priv->blank_command? "can":"cannot");
+
 	g_free (data);
 }
 
@@ -991,7 +1015,7 @@ brasero_medium_init_caps (BraseroMedium *self,
 	if (priv->info & BRASERO_MEDIUM_CD) {
 		/* we have to do both */
 		res = brasero_medium_test_simulate_CD_SAO (self, handle, code);
-		if (res == BRASERO_SCSI_OK)
+		if (res)
 			brasero_medium_test_simulate_CD_TAO (self, handle, code);
 	}
 	else
@@ -1002,7 +1026,7 @@ brasero_medium_init_caps (BraseroMedium *self,
 			  priv->dummy_sao,
 			  priv->burnfree);
 
-	if (res == BRASERO_SCSI_OK)
+	if (res)
 		return;
 
 	/* it didn't work out as expected use fallback */
@@ -2204,6 +2228,12 @@ brasero_medium_get_contents (BraseroMedium *self,
 	else if (info->status == BRASERO_SCSI_DISC_INCOMPLETE) {
 		if (!BRASERO_MEDIUM_RANDOM_WRITABLE (priv->info)) {
 			priv->info |= BRASERO_MEDIUM_APPENDABLE;
+
+			/* This is just to make sure the disc is in a correct
+			 * state as I saw some drive being flagged as unformatted
+			 * appendable */
+			priv->info &= ~(BRASERO_MEDIUM_UNFORMATTED);
+
 			BRASERO_MEDIA_LOG ("Appendable media");
 
 			priv->first_open_track = BRASERO_FIRST_TRACK_IN_LAST_SESSION (info);
@@ -2893,21 +2923,21 @@ static gpointer
 brasero_medium_probe_thread (gpointer self)
 {
 	gint counter = 0;
-	const gchar *path;
+	const gchar *device;
 	BraseroScsiErrCode code;
 	BraseroMediumPrivate *priv;
 	BraseroDeviceHandle *handle;
 
 	priv = BRASERO_MEDIUM_PRIVATE (self);
-	path = brasero_drive_get_device (priv->drive);
 
 	priv->info = BRASERO_MEDIUM_BUSY;
 
 	/* the drive might be busy (a burning is going on) so we don't block
 	 * but we re-try to open it every second */
-	BRASERO_MEDIA_LOG ("Trying to open device %s", path);
+	device = brasero_drive_get_block_device (priv->drive);
+	BRASERO_MEDIA_LOG ("Trying to open device %s", device);
 
-	handle = brasero_device_handle_open (path, FALSE, &code);
+	handle = brasero_device_handle_open (device, FALSE, &code);
 	while (!handle && counter <= BRASERO_MEDIUM_OPEN_ATTEMPTS) {
 		sleep (1);
 
@@ -2917,7 +2947,7 @@ brasero_medium_probe_thread (gpointer self)
 		}
 
 		counter ++;
-		handle = brasero_device_handle_open (path, FALSE, &code);
+		handle = brasero_device_handle_open (device, FALSE, &code);
 	}
 
 	if (priv->probe_cancelled) {
@@ -3000,6 +3030,8 @@ brasero_medium_finalize (GObject *object)
 
 	priv = BRASERO_MEDIUM_PRIVATE (object);
 
+	BRASERO_MEDIA_LOG ("Finalizing Medium object");
+
 	if (priv->probe) {
 		priv->probe_cancelled = TRUE;
 		g_thread_join (priv->probe);
@@ -3125,6 +3157,14 @@ brasero_medium_class_init (BraseroMediumClass *klass)
 	                                                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 }
 
+
+/* This function is not public API yet because it was too
+ * late; so use it internally for now. It's mainly for 
+ * convenience.*/
+gboolean
+brasero_drive_can_write_media (BraseroDrive *drive,
+                               BraseroMedia media);
+
 /**
  * brasero_medium_can_be_written:
  * @medium: #BraseroMedium
@@ -3139,59 +3179,13 @@ gboolean
 brasero_medium_can_be_written (BraseroMedium *medium)
 {
 	BraseroMediumPrivate *priv;
-	BraseroDriveCaps caps;
 
 	g_return_val_if_fail (medium != NULL, FALSE);
 	g_return_val_if_fail (BRASERO_IS_MEDIUM (medium), FALSE);
 
 	priv = BRASERO_MEDIUM_PRIVATE (medium);
 
-	if (!(priv->info & BRASERO_MEDIUM_REWRITABLE)
-	&&   (priv->info & BRASERO_MEDIUM_CLOSED))
-		return FALSE;
-
-	if (priv->info & BRASERO_MEDIUM_FILE)
-		return FALSE;
-
-	caps = brasero_drive_get_caps (priv->drive);
-	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_CDR))
-		return (caps & BRASERO_DRIVE_CAPS_CDR) != 0;
-
-	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDR))
-		return (caps & BRASERO_DRIVE_CAPS_DVDR) != 0;
-
-	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDR_PLUS))
-		return (caps & BRASERO_DRIVE_CAPS_DVDR_PLUS) != 0;
-
-	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_CDRW))
-		return (caps & BRASERO_DRIVE_CAPS_CDRW) != 0;
-
-	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW))
-		return (caps & BRASERO_DRIVE_CAPS_DVDRW) != 0;
-
-	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_RESTRICTED))
-		return (caps & BRASERO_DRIVE_CAPS_DVDRW) != 0;
-
-	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_PLUS))
-		return (caps & BRASERO_DRIVE_CAPS_DVDRW_PLUS) != 0;
-
-	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDR_PLUS_DL))
-		return (caps & BRASERO_DRIVE_CAPS_DVDR_PLUS_DL) != 0;
-
-	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_PLUS_DL))
-		return (caps & BRASERO_DRIVE_CAPS_DVDRW_PLUS_DL) != 0;
-
-	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVD_RAM))
-		return (caps & BRASERO_DRIVE_CAPS_DVDRAM) != 0;
-
-	/* All types of BD-R */
-	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_BD|BRASERO_MEDIUM_WRITABLE))
-		return (caps & BRASERO_DRIVE_CAPS_BDR) != 0;
-
-	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_BDRE))
-		return (caps & BRASERO_DRIVE_CAPS_BDRW) != 0;
-
-	return FALSE;
+	return brasero_drive_can_write_media (priv->drive, priv->info);
 }
 
 /**
@@ -3208,7 +3202,6 @@ gboolean
 brasero_medium_can_be_rewritten (BraseroMedium *medium)
 {
 	BraseroMediumPrivate *priv;
-	BraseroDriveCaps caps;
 
 	g_return_val_if_fail (medium != NULL, FALSE);
 	g_return_val_if_fail (BRASERO_IS_MEDIUM (medium), FALSE);
@@ -3219,30 +3212,29 @@ brasero_medium_can_be_rewritten (BraseroMedium *medium)
 	||   (priv->info & BRASERO_MEDIUM_FILE))
 		return FALSE;
 
-	caps = brasero_drive_get_caps (priv->drive);
-	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_CDRW))
-		return (caps & BRASERO_DRIVE_CAPS_CDRW) != 0;
+	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_CDRW)
+	||  BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW))
+		return priv->blank_command != 0;
 
-	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW))
-		return (caps & BRASERO_DRIVE_CAPS_DVDRW) != 0;
-
-	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_RESTRICTED))
-		return (caps & BRASERO_DRIVE_CAPS_DVDRW) != 0;
-
-	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_PLUS))
-		return (caps & BRASERO_DRIVE_CAPS_DVDRW_PLUS) != 0;
-
-	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_PLUS_DL))
-		return (caps & BRASERO_DRIVE_CAPS_DVDRW_PLUS_DL) != 0;
-
-	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVD_RAM))
-		return (caps & BRASERO_DRIVE_CAPS_DVDRAM) != 0;
-
-	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_BDRE))
-		return (caps & BRASERO_DRIVE_CAPS_BDRW) != 0;
+	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_RESTRICTED)
+	||  BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_PLUS)
+	||  BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_PLUS_DL)
+	||  BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVD_RAM)
+	||  BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_BDRE))
+		return priv->write_command != 0;
 
 	return FALSE;
 }
+
+/**
+ * brasero_medium_can_use_dummy_for_sao:
+ * @medium: #BraseroMedium
+ *
+ * Gets whether the medium supports doing a test write with SAO on.
+ *
+ * Return value: a #gboolean. TRUE if the medium can use SAO write mode during a test write, FALSE otherwise.
+ *
+ **/
 gboolean
 brasero_medium_can_use_dummy_for_sao (BraseroMedium *medium)
 {
@@ -3254,6 +3246,15 @@ brasero_medium_can_use_dummy_for_sao (BraseroMedium *medium)
 	return priv->dummy_sao;
 }
 
+/**
+ * brasero_medium_can_use_dummy_for_tao:
+ * @medium: #BraseroMedium
+ *
+ * Gets whether the medium supports doing a test write with TAO on.
+ *
+ * Return value: a #gboolean. TRUE if the medium can use TAO write mode during a test write, FALSE otherwise.
+ *
+ **/
 gboolean
 brasero_medium_can_use_dummy_for_tao (BraseroMedium *medium)
 {
@@ -3265,6 +3266,15 @@ brasero_medium_can_use_dummy_for_tao (BraseroMedium *medium)
 	return priv->dummy_tao;
 }
 
+/**
+ * brasero_medium_can_use_burnfree:
+ * @medium: #BraseroMedium
+ *
+ * Gets whether the medium supports any burnfree technology.
+ *
+ * Return value: a #gboolean. TRUE if the medium can use any burnfree technology, FALSE otherwise.
+ *
+ **/
 gboolean
 brasero_medium_can_use_burnfree (BraseroMedium *medium)
 {
