@@ -57,7 +57,7 @@ G_DEFINE_TYPE (BraseroVolume, brasero_volume, BRASERO_TYPE_MEDIUM);
  * brasero_volume_get_gvolume:
  * @volume: #BraseroVolume
  *
- * Gets the corresponding #GVolume for @Medium.
+ * Gets the corresponding #GVolume for @volume.
  *
  * Return value: a #GVolume *.
  *
@@ -233,6 +233,11 @@ brasero_volume_umount (BraseroVolume *volume,
 	if (!gvolume)
 		return TRUE;
 
+	if (g_cancellable_is_cancelled (priv->cancel)) {
+		BRASERO_MEDIA_LOG ("Resetting GCancellable object");
+		g_cancellable_reset (priv->cancel);
+	}
+
 	result = brasero_gio_operation_umount (gvolume,
 					       priv->cancel,
 					       wait,
@@ -278,6 +283,11 @@ brasero_volume_mount (BraseroVolume *volume,
 	if (!gvolume)
 		return TRUE;
 
+	if (g_cancellable_is_cancelled (priv->cancel)) {
+		BRASERO_MEDIA_LOG ("Resetting GCancellable object");
+		g_cancellable_reset (priv->cancel);
+	}
+
 	result = brasero_gio_operation_mount (gvolume,
 					      parent_window,
 					      priv->cancel,
@@ -303,7 +313,10 @@ brasero_volume_cancel_current_operation (BraseroVolume *volume)
 	g_return_if_fail (volume != NULL);
 	g_return_if_fail (BRASERO_IS_VOLUME (volume));
 
-	priv = BRASERO_VOLUME_PRIVATE (volume);	
+	priv = BRASERO_VOLUME_PRIVATE (volume);
+
+	BRASERO_MEDIA_LOG ("Cancelling volume operation");
+
 	g_cancellable_cancel (priv->cancel);
 }
 
@@ -402,23 +415,22 @@ last_chance:
 	type = brasero_medium_get_type_string (BRASERO_MEDIUM (volume));
 	name = NULL;
 	if (media & BRASERO_MEDIUM_BLANK) {
-		/* NOTE for translators: the first %s is the disc type. */
-		name = g_strdup_printf (_("Blank %s"), type);
+		/* NOTE for translators: the first %s is the disc type and Blank is an adjective. */
+		name = g_strdup_printf (_("Blank disc (%s)"), type);
 	}
 	else if (BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_HAS_AUDIO|BRASERO_MEDIUM_HAS_DATA)) {
 		/* NOTE for translators: the first %s is the disc type. */
-		name = g_strdup_printf (_("Audio and data %s"), type);
+		name = g_strdup_printf (_("Audio and data disc (%s)"), type);
 	}
 	else if (media & BRASERO_MEDIUM_HAS_AUDIO) {
 		/* NOTE for translators: the first %s is the disc type. */
-		name = g_strdup_printf (_("Audio %s"), type);
+		name = g_strdup_printf (_("Audio disc (%s)"), type);
 	}
 	else if (media & BRASERO_MEDIUM_HAS_DATA) {
 		/* NOTE for translators: the first %s is the disc type. */
-		name = g_strdup_printf (_("Data %s"), type);
+		name = g_strdup_printf (_("Data disc (%s)"), type);
 	}
 	else {
-		/* NOTE for translators: the first %s is the disc type. */
 		name = g_strdup (type);
 	}
 
@@ -441,6 +453,7 @@ brasero_volume_finalize (GObject *object)
 
 	priv = BRASERO_VOLUME_PRIVATE (object);
 
+	BRASERO_MEDIA_LOG ("Finalizing Volume object");
 	if (priv->cancel) {
 		g_cancellable_cancel (priv->cancel);
 		g_object_unref (priv->cancel);
@@ -459,22 +472,3 @@ brasero_volume_class_init (BraseroVolumeClass *klass)
 
 	object_class->finalize = brasero_volume_finalize;
 }
-
-/**
- * This is private API defined in brasero-drive.c
- */
-
-BraseroVolume *
-brasero_volume_new (BraseroDrive *drive,
-		    const gchar *udi)
-{
-	BraseroVolume *volume;
-
-	g_return_val_if_fail (drive != NULL, NULL);
-	volume = g_object_new (BRASERO_TYPE_VOLUME,
-			       "drive", drive,
-			       "udi", udi,
-			       NULL);
-
-	return volume;
-}
