@@ -1,24 +1,28 @@
-/***************************************************************************
- *            burn-sum.c
- *
- *  ven aoû  4 19:46:34 2006
- *  Copyright  2006  Rouquier Philippe
- *  brasero-app@wanadoo.fr
- ***************************************************************************/
-
+/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /*
- *  Brasero is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ * Libbrasero-burn
+ * Copyright (C) Philippe Rouquier 2005-2009 <bonfire-app@wanadoo.fr>
  *
- *  Brasero is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ * Libbrasero-burn is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to:
+ * The Libbrasero-burn authors hereby grant permission for non-GPL compatible
+ * GStreamer plugins to be used and distributed together with GStreamer
+ * and Libbrasero-burn. This permission is above and beyond the permissions granted
+ * by the GPL license by which Libbrasero-burn is covered. If you modify this code
+ * you may extend this exception to your version of the code, but you are not
+ * obligated to do so. If you do not wish to do so, delete this exception
+ * statement from your version.
+ * 
+ * Libbrasero-burn is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Library General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to:
  * 	The Free Software Foundation, Inc.,
  * 	51 Franklin Street, Fifth Floor
  * 	Boston, MA  02110-1301, USA.
@@ -382,12 +386,14 @@ brasero_checksum_files_merge_with_former_session (BraseroChecksumFiles *self,
 	BraseroDeviceHandle *dev_handle;
 	BraseroVolFileHandle *handle;
 	BraseroBurnResult result;
+	BraseroDrive *burner;
+	BraseroMedium *medium;
 	BraseroVolFile *file;
 	BraseroTrack *track;
 	gchar buffer [2048];
 	BraseroVolSrc *vol;
 	goffset start_block;
-	gchar *device;
+	const gchar *device;
 
 	priv = BRASERO_CHECKSUM_FILES_PRIVATE (self);
 
@@ -403,9 +409,11 @@ brasero_checksum_files_merge_with_former_session (BraseroChecksumFiles *self,
 		return result;
 
 	/* try every file and make sure they are of the same type */
-	brasero_job_get_device (BRASERO_JOB (self), &device);
+	medium = NULL;
+	brasero_job_get_medium (BRASERO_JOB (self), &medium);
+	burner = brasero_medium_get_drive (medium);
+	device = brasero_drive_get_block_device (burner);
 	dev_handle = brasero_device_handle_open (device, FALSE, NULL);
-	g_free (device);
 
 	vol = brasero_volume_source_open_device_handle (dev_handle, error);
 	file = brasero_volume_get_file (vol,
@@ -824,7 +832,7 @@ brasero_checksum_files_check_files (BraseroChecksumFiles *self,
 	if (!brasero_medium_get_last_data_track_address (medium, NULL, &start_block))
 		return BRASERO_BURN_ERR;
 
-	device = brasero_drive_get_device (brasero_medium_get_drive (medium));
+	device = brasero_drive_get_block_device (brasero_medium_get_drive (medium));
 	dev_handle = brasero_device_handle_open (device, FALSE, NULL);
 	vol = brasero_volume_source_open_device_handle (dev_handle, error);
 
