@@ -50,8 +50,6 @@
 #include "brasero-app.h"
 #include "brasero-project-manager.h"
 
-#include "burn-basics.h"
-
 #include "brasero-session-cfg.h"
 #include "brasero-tags.h"
 #include "brasero-track.h"
@@ -778,10 +776,10 @@ brasero_data_disc_name_collision_cb (BraseroTrackDataCfg *project,
 	priv = BRASERO_DATA_DISC_PRIVATE (self);
 
 	if (priv->always_replace)
-		return TRUE;
+		return FALSE;
 
 	if (priv->never_replace)
-		return FALSE;
+		return TRUE;
 
 	/* Translators: %s is the name of the file */
 	string = g_strdup_printf (_("Do you really want to replace \"%s\"?"), name);
@@ -970,7 +968,7 @@ brasero_disc_disc_session_import_response_cb (GtkButton *button,
 	g_signal_handlers_unblock_by_func (action, brasero_data_disc_import_session_cb, self);
 }
 
-static void
+static GtkAction *
 brasero_data_disc_import_button_new (BraseroDataDisc *self,
 				     BraseroMedium *medium)
 {
@@ -987,7 +985,7 @@ brasero_data_disc_import_button_new (BraseroDataDisc *self,
 	priv = BRASERO_DATA_DISC_PRIVATE (self);
 
 	if (!priv->manager)
-		return;
+		return NULL;
 
 	action_name = g_strdup_printf ("Import_%s", BRASERO_MEDIUM_GET_UDI (medium));
 
@@ -1021,7 +1019,7 @@ brasero_data_disc_import_button_new (BraseroDataDisc *self,
 	action = gtk_action_group_get_action (priv->import_group, action_name);
 	if (!action) {
 		g_free (action_name);
-		return;
+		return NULL;
 	}
 
 	g_object_ref (medium);
@@ -1061,6 +1059,7 @@ brasero_data_disc_import_button_new (BraseroDataDisc *self,
 
 	g_free (description);
 	g_free (action_name);
+	return action;
 }
 
 static void
@@ -1302,6 +1301,7 @@ static BraseroDiscResult
 brasero_data_disc_set_track (BraseroDataDisc *disc,
 			     BraseroTrackDataCfg *track)
 {
+	BraseroMedium *loaded_medium;
 	BraseroDataDiscPrivate *priv;
 	BraseroStatus *status;
 	GtkWidget *message;
@@ -1325,6 +1325,7 @@ brasero_data_disc_set_track (BraseroDataDisc *disc,
 		gtk_action_group_set_visible (priv->disc_group, TRUE);
 
 	/* Now let's take care of all the available sessions */
+	loaded_medium = brasero_track_data_cfg_get_current_medium (track);
 	if (!priv->import_group) {
 		GSList *iter;
 		GSList *list;
@@ -1338,9 +1339,15 @@ brasero_data_disc_set_track (BraseroDataDisc *disc,
 		list = brasero_track_data_cfg_get_available_media (priv->project);
 		for (iter = list; iter; iter = iter->next) {
 			BraseroMedium *medium;
+			GtkAction *action;
 
 			medium = iter->data;
-			brasero_data_disc_import_button_new (BRASERO_DATA_DISC (disc), medium);
+			action = brasero_data_disc_import_button_new (BRASERO_DATA_DISC (disc), medium);
+			if (!action)
+				continue;
+
+			if (medium == loaded_medium)
+				gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), TRUE);
 		}
 		g_slist_foreach (list, (GFunc) g_object_unref, NULL);
 		g_slist_free (list);
@@ -1485,14 +1492,15 @@ brasero_data_disc_unset_track (BraseroDataDisc *disc)
 	/* Remove each button for every available session that can be imported */
 	if (priv->import_group) {
 		GList *actions;
+		GList *iter;
 
 		actions = gtk_action_group_list_actions (priv->import_group);
-		for (; actions; actions = actions->next) {
+		for (iter = actions; iter; iter = iter->next) {
 			BraseroMedium *medium;
 			GtkAction *action;
 			int merge_id;
 
-			action = actions->data;
+			action = iter->data;
 
 			/* We reffed the medium associated with the action */
 			medium = g_object_get_data (G_OBJECT (action), BRASERO_DATA_DISC_MEDIUM);
@@ -1600,6 +1608,9 @@ brasero_data_disc_set_session_contents (BraseroDisc *self,
 						NULL);
 		brasero_data_disc_set_track (BRASERO_DATA_DISC (self),
 					     BRASERO_TRACK_DATA_CFG (data_track));
+
+		/* NOTE: that track was reffed in brasero_data_disc_set_track () */
+		g_object_unref (data_track);
 	}
 	else for (; tracks; tracks = tracks->next) {
 		BraseroTrack *track;
@@ -2343,6 +2354,11 @@ brasero_data_disc_finalize (GObject *object)
 
 	priv = BRASERO_DATA_DISC_PRIVATE (object);
 
+	if (priv->project) {
+		g_object_unref (priv->project);
+		priv->project = NULL;
+	}
+
 	if (priv->button_size) {
 		g_object_unref (priv->button_size);
 		priv->button_size = NULL;
