@@ -33,6 +33,7 @@
 #include <gtk/gtk.h>
 
 #include "brasero-medium.h"
+#include "brasero-project-parse.h"
 #include "brasero-project-type-chooser.h"
 
 G_BEGIN_DECLS
@@ -58,19 +59,19 @@ typedef struct {
 GType brasero_project_manager_get_type ();
 GtkWidget *brasero_project_manager_new ();
 
-void
-brasero_project_manager_set_oneshot (BraseroProjectManager *manager,
-				     gboolean oneshot);
 
 void
 brasero_project_manager_audio (BraseroProjectManager *manager,
-			       GSList *uris);
+			       GSList *uris,
+			       gboolean burn);
 void
 brasero_project_manager_data (BraseroProjectManager *manager,
-			      GSList *uris);
+			      GSList *uris,
+			      gboolean burn);
 void
 brasero_project_manager_video (BraseroProjectManager *manager,
-			       GSList *uris);
+			       GSList *uris,
+			       gboolean burn);
 void
 brasero_project_manager_copy (BraseroProjectManager *manager,
 			      const gchar *device,
@@ -78,17 +79,12 @@ brasero_project_manager_copy (BraseroProjectManager *manager,
 void
 brasero_project_manager_iso (BraseroProjectManager *manager,
 			     const gchar *uri);
-void
-brasero_project_manager_burn_project (BraseroProjectManager *manager,
-				      const gchar *uri);
-
-BraseroProjectType
-brasero_project_manager_open_playlist (BraseroProjectManager *manager,
-				       const gchar *uri);
 
 BraseroProjectType
 brasero_project_manager_open_project (BraseroProjectManager *manager,
-				      const gchar *uri);
+				      const gchar *uri,
+				      gboolean playlist,
+				      gboolean burn);
 
 BraseroProjectType
 brasero_project_manager_open_by_mime (BraseroProjectManager *manager,
