@@ -29,6 +29,8 @@
 
 #include <gtk/gtk.h>
 
+#include "brasero-units.h"
+
 #include "brasero-session-cfg.h"
 #include "brasero-tags.h"
 #include "brasero-track-stream-cfg.h"
@@ -102,6 +104,30 @@ static gint
 brasero_video_tree_model_iter_n_children (GtkTreeModel *model,
 					  GtkTreeIter *iter)
 {
+	if (!iter) {
+		guint num = 0;
+		GSList *iter;
+		GSList * tracks;
+		BraseroVideoTreeModelPrivate *priv;
+
+		priv = BRASERO_VIDEO_TREE_MODEL_PRIVATE (model);
+
+		/* This is a special case in which we return the number
+		 * of rows that are in the model. */
+		tracks = brasero_burn_session_get_tracks (BRASERO_BURN_SESSION (priv->session));
+		for (iter = tracks; iter; iter = iter->next) {
+			BraseroTrackStream *track;
+
+			track = iter->data;
+			num ++;
+
+			if (brasero_track_stream_get_gap (track) > 0)
+				num ++;
+		}
+
+		return num;
+	}
+
 	return 0;
 }
 
@@ -149,11 +175,17 @@ brasero_video_tree_model_get_value (GtkTreeModel *model,
 
 	if (GPOINTER_TO_INT (iter->user_data2) == BRASERO_STREAM_ROW_GAP) {
 		switch (column) {
+		case BRASERO_VIDEO_TREE_MODEL_WEIGHT:
+			g_value_init (value, PANGO_TYPE_STYLE);
+			g_value_set_enum (value, PANGO_WEIGHT_BOLD);
+			return;
+		case BRASERO_VIDEO_TREE_MODEL_STYLE:
+			g_value_init (value, PANGO_TYPE_STYLE);
+			g_value_set_enum (value, PANGO_STYLE_ITALIC);
+			return;
 		case BRASERO_VIDEO_TREE_MODEL_NAME:
-			text = g_strdup_printf ("<i><b>%s</b></i>", _("Pause"));
 			g_value_init (value, G_TYPE_STRING);
-			g_value_set_string (value, text);
-			g_free (text);
+			g_value_set_string (value, _("Pause"));
 			break;
 		case BRASERO_VIDEO_TREE_MODEL_ICON_NAME:
 			g_value_init (value, G_TYPE_STRING);
@@ -193,12 +225,21 @@ brasero_video_tree_model_get_value (GtkTreeModel *model,
 	}
 
 	switch (column) {
+	case BRASERO_VIDEO_TREE_MODEL_WEIGHT:
+		g_value_init (value, PANGO_TYPE_STYLE);
+		g_value_set_enum (value, PANGO_WEIGHT_NORMAL);
+		return;
+	case BRASERO_VIDEO_TREE_MODEL_STYLE:
+		g_value_init (value, PANGO_TYPE_STYLE);
+		g_value_set_enum (value, PANGO_STYLE_NORMAL);
+		return;
 	case BRASERO_VIDEO_TREE_MODEL_NAME:
 		g_value_init (value, G_TYPE_STRING);
 
 		string = brasero_track_tag_lookup_string (track, BRASERO_TRACK_STREAM_TITLE_TAG);
-		if (string)
+		if (string) {
 			g_value_set_string (value, string);
+		}
 		else {
 			gchar *uri;
 			gchar *name;
@@ -225,7 +266,7 @@ brasero_video_tree_model_get_value (GtkTreeModel *model,
 		g_value_init (value, G_TYPE_STRING);
 
 		string = brasero_track_tag_lookup_string (track, BRASERO_TRACK_STREAM_ARTIST_TAG);
-		if (string)
+		if (string) 
 			g_value_set_string (value, string);
 
 		return;
@@ -608,6 +649,12 @@ brasero_video_tree_model_get_column_type (GtkTreeModel *model,
 	case BRASERO_VIDEO_TREE_MODEL_IS_GAP:
 		return G_TYPE_STRING;
 
+	case BRASERO_VIDEO_TREE_MODEL_WEIGHT:
+		return PANGO_TYPE_WEIGHT;
+
+	case BRASERO_VIDEO_TREE_MODEL_STYLE:
+		return PANGO_TYPE_STYLE;
+
 	default:
 		break;
 	}
